﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace ccc4
{


    // 正4面体をグリッド表示する時に
    // 正4面体クラスのDrawグリッドで1つずつ表示すると遅いので
    // グリッド全てを1度に表示する。
    class 正4面体グリッド : 基本Model
    {

        VertexDeclaration               v宣言 = null;   // Declaration : 宣言

        VertexPositionNormalColor[]     Vertices;  // Vertices:頂点 (本体)
        
        VertexBuffer                    vバッファ;      // Vertex buffer to define a triangle.

        // --------------------------------------
        public 正4面体グリッド()
        {
        }



        int m_gSize = 0; // グリッドの個数

        // --------------------------------------
        // len   正4面体の辺の長さ (float)
        // gSize グリッドの大きさ  (int) 例:8 * 8 * 8
        // g間隔 グリッド間の間隔  (float)
        public void Initialize(GraphicsDeviceManager graphics, float len, int gSize, float g間隔 )
        {
            int iVsize = 12 * (gSize * gSize * gSize);  // 配列長
            m_gSize = (gSize * gSize * gSize); // グリッドの個数


            // Boxグリッド表示(8*8*8)
            float g間隔調整 = g間隔 * ((gSize - 1) * 0.5f);   // グリッド中心を原点に移動


            v宣言 = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionNormalColor.VertexElements);

            vバッファ = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionNormalColor), iVsize, BufferUsage.WriteOnly);

            Vertices = new VertexPositionNormalColor[iVsize];   // テクスチャ無し

            Color Col = Color.MidnightBlue; // 表示色

            float len2 = len / 2f;

            float t = ( (float)Math.Sqrt(2) / 2 ) * len2;  // 正四面体の高さ

            Vector3 p0 = new Vector3(-len2,  t,     0 );
            Vector3 p1 = new Vector3( len2,  t,     0 );
            Vector3 p2 = new Vector3( 0,    -t, -len2 );
            Vector3 p3 = new Vector3( 0,    -t,  len2 );

            Vector3 Normal = Vector3.Zero;

            Vector3 前Normal;
            Vector3 後Normal;
            Vector3 左Normal;
            Vector3 右Normal;

            int os = 0;   // offset

            Vector3 wk_pos;

            for (int i = 0; i < gSize; i++)
                for (int j = 0; j < gSize; j++)
                    for (int k = 0; k < gSize; k++)
                    {
                        wk_pos = new Vector3(   (g間隔 * i) - g間隔調整,
                                                (g間隔 * j) - g間隔調整,
                                                (g間隔 * k) - g間隔調整 );

                        // 左回り(反時計回り)に設定

                        // VertexPositionNormalColor
                        // 前
                        Vertices[os + 0] = new VertexPositionNormalColor(p0 + wk_pos, Normal, Col);
                        Vertices[os + 1] = new VertexPositionNormalColor(p3 + wk_pos, Normal, Col);
                        Vertices[os + 2] = new VertexPositionNormalColor(p1 + wk_pos, Normal, Col);

                        // 後
                        Vertices[os + 3] = new VertexPositionNormalColor(p1 + wk_pos, Normal, Col);
                        Vertices[os + 4] = new VertexPositionNormalColor(p2 + wk_pos, Normal, Col);
                        Vertices[os + 5] = new VertexPositionNormalColor(p0 + wk_pos, Normal, Col);

                        // 左
                        Vertices[os + 6] = new VertexPositionNormalColor(p3 + wk_pos, Normal, Col);
                        Vertices[os + 7] = new VertexPositionNormalColor(p2 + wk_pos, Normal, Col);
                        Vertices[os + 8] = new VertexPositionNormalColor(p1 + wk_pos, Normal, Col);

                        // 右
                        Vertices[os + 9]  = new VertexPositionNormalColor(p2 + wk_pos, Normal, Col);
                        Vertices[os + 10] = new VertexPositionNormalColor(p3 + wk_pos, Normal, Col);
                        Vertices[os + 11] = new VertexPositionNormalColor(p0 + wk_pos, Normal, Col);



                        // ================================================================
                        // 法線を求め、Verticesを再定義する。
                        int 法線os = os; // offset
                        前Normal = NormalFromTriangle_法線設定2(
                            Vertices[0 + 法線os].Position,
                            Vertices[1 + 法線os].Position,
                            Vertices[2 + 法線os].Position);

                        法線os += 3;
                        後Normal = NormalFromTriangle_法線設定2(
                            Vertices[0 + 法線os].Position,
                            Vertices[1 + 法線os].Position,
                            Vertices[2 + 法線os].Position);

                        法線os += 3;
                        左Normal = NormalFromTriangle_法線設定2(
                            Vertices[0 + 法線os].Position,
                            Vertices[1 + 法線os].Position,
                            Vertices[2 + 法線os].Position);

                        法線os += 3;
                        右Normal = NormalFromTriangle_法線設定2(
                            Vertices[0 + 法線os].Position,
                            Vertices[1 + 法線os].Position,
                            Vertices[2 + 法線os].Position);

                        // 前
                        Vertices[os + 0].Normal = 前Normal;
                        Vertices[os + 1].Normal = 前Normal;
                        Vertices[os + 2].Normal = 前Normal;

                        // 後
                        Vertices[os + 3].Normal = 後Normal;
                        Vertices[os + 4].Normal = 後Normal;
                        Vertices[os + 5].Normal = 後Normal;

                        // 左
                        Vertices[os + 6].Normal = 左Normal;
                        Vertices[os + 7].Normal = 左Normal;
                        Vertices[os + 8].Normal = 左Normal;

                        // 右
                        Vertices[os + 9].Normal = 右Normal;
                        Vertices[os + 10].Normal = 右Normal;
                        Vertices[os + 11].Normal = 右Normal;

                        os += 12;
                    } // for end ===========================================================

            vバッファ.SetData<VertexPositionNormalColor>(Vertices);
        }



        // --------------------------------------



        // --------------------------------------
        // BasicEffect effect,     // ライティング+テクスチャ 
        // テクスチャが無い場合、マテリアルで色設定
        // Fogの設定
        // 半透明の設定
        public void Draw(GraphicsDeviceManager graphics,
                            BasicEffect effect,     // ライティング+テクスチャ 
                            Matrix view,
                            Matrix proj)
        {
            effect.World        = this.単位行列 * Matrix.CreateTranslation(this.pos);
            effect.View         = view;
            effect.Projection   = proj;

            effect.TextureEnabled = false;

            // ---
            /*
                        effect.AmbientLightColor = new Vector3(1,0,0);  // 効果:強い
                        effect.SpecularColor = new Vector3(1, 0, 0);    // 弱い
                        effect.DiffuseColor = new Vector3(1, 0, 0); // 強い
                        effect.EmissiveColor = new Vector3(1, 0, 0);    // 弱い
            */

            // ライトによる表示色設定
//          effect.AmbientLightColor = new Vector3(0.1f, 0.1f, 0.2f);  // 効果:強い

            // アンビエント光 (環境光)
            // effect.AmbientLightColor = new Vector3(1, 0, 0);
            // アンビエント光は、光が拡散し、すべてのオブジェクトを一様に照らすライト。

            /*
            スペキュラ光
            effect.SpecularColor = new Vector3(1, 0, 0);
            specular light
            鏡面反射光
            物体に光があたったときに、最も強くその光を反射している部分の光の総称。
            */

            /*
            デフューズ光
            effect.DiffuseColor = new Vector3(1, 0, 0);
            diffuse light
            拡散反射光
            物体にあたった光はその内部に取り込まれ、反射を繰り返して拡散光となってふたたび外に出る。この拡散光の総称。
            */


            /*
            effect.EmissiveColor = new Vector3(1, 0, 0);
            発光による色
            それ自体発光している物体の発光色を設定します。 
            */


            // Fog : 視点からの距離では無く、座標原点からの距離でFogがかかる
            //       使用方法研究
            // FogColorは背景色にする。
            /*
                        effect.FogColor = new Vector3(0, 0, 0);
                        effect.FogStart =   50f;
                        effect.FogEnd   =  100f;
                        effect.FogEnabled = true;
            */

            graphics.GraphicsDevice.VertexDeclaration = this.v宣言;
            graphics.GraphicsDevice.Vertices[0].SetSource(vバッファ, 0, VertexPositionNormalColor.Size);

            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 4 * m_gSize);

                pass.End();
            }
            effect.End();

            // ライトによる表示色設定:元に戻す
//          effect.AmbientLightColor = new Vector3(0.75f, 0.75f, 0.75f);

        } // Draw2 end







    
    }


}
