﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;


namespace ccc4
{


    // **********************
    // http://memeplex.blog.shinobi.jp/Entry/55/
    // かんたんXNA　その３０　カスタム頂点 より
    public struct VertexPositionNormalColor
    {
        public Vector3 Position;
        public Vector3 Normal;
        public Color Color;



        public VertexPositionNormalColor(Vector3 position, Vector3 normal, Color color)
        {
            this.Position = position;
            this.Normal = normal;
            this.Color = color;
        }

        public static readonly VertexElement[] VertexElements = {
            // Position
            new VertexElement(
                0,
                0,
                VertexElementFormat.Vector3,
                VertexElementMethod.Default,
                VertexElementUsage.Position,
                0
            ),

            // Normal
            new VertexElement(
                0, 
                sizeof(float) * 3,
                VertexElementFormat.Vector3,
                VertexElementMethod.Default,
                VertexElementUsage.Normal,
                0
            ),

            // Color
            new VertexElement(
                0,
                sizeof(float)*(3 + 3),
                VertexElementFormat.Color,
                VertexElementMethod.Default,
                VertexElementUsage.Color,
                0
            )
        };

        // 頂点バッファを使うのなら、頂点のサイズを表すメンバが必要
        public static readonly int Size = sizeof(float) * (3 + 3) + sizeof(byte) * 4;
        // sizeof(float) * (3 + 3) : 頂点+法線
        // sizeof(byte) * 4 : Color rgba

    }

    // **********************

    public struct Face
    {
        public int A, B, C;

        public void Set(int a, int b, int c)
        {
            this.A = a;
            this.B = b;
            this.C = c;
        }
    }


    // ========================================================================
    // カスタム頂点VertexPositionNormalColor を使用した球
    // Simple Program 3 http://www.gameprojects.com/project/?id=942a7cc022
    public class 色球 : 基本Model
    {
        VertexDeclaration           decl_頂点宣言;      // 頂点宣言
        VertexPositionNormalColor[] Vertices_頂点配列;  // Vertices:頂点 (本体)
        VertexBuffer                VB_頂点バッファ;

        BoundingSphere 制限球;

        // --------------------------------------
        public 色球()
        {
        }






        // 赤道を表示する為はに円周数を6 + (4*x)とする。
        // 6,10,14,18,22,26,30,...122...254
        // ポリゴン毎に色を変える。
        public void Initialize(GraphicsDeviceManager graphics, int 円周, float 半径)
        {
            // cube本体
            decl_頂点宣言 = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionNormalColor.VertexElements);

            this.制限球 = new BoundingSphere(Vector3.Zero, 半径);

            int segment_円周_Count = 円周; // 円周分割数を設定

            // 円周範囲チェック
            if (円周 < 4)
            {
                segment_円周_Count = 4; // 最小円周分割数を設定
            }
            if (円周 > 1024)
            {
                segment_円周_Count = 1024; // 最大円周分割数を設定
            }


            int totalRows_円数 = segment_円周_Count / 2 - 1; //The number of rows in the sphere, not counting top and bottom verts

            int totalVerts = totalRows_円数 * segment_円周_Count + 2;
            int totalFaces = totalRows_円数 * segment_円周_Count * 2;   // 面(ポリゴン)数                       


            Vertices_頂点配列 = new VertexPositionNormalColor[totalFaces * 3]; // 頂点数 = ポリゴン数 * 3


            this.oVertices_頂点配列 = new Vector3[totalVerts];

            int ix, jx, kx;		// Index variables
            int na, nb, nc, nd; // Temporary integers

            int vertCount = 0;	// Counters for the number of vertices

            float a, b, c;					//Temporary floats
            float alt, secRad, secAng;		//Altitude, radial secant, angular secant

            float delta = 2.0f * MathHelper.Pi / segment_円周_Count;
            float delta2 = delta;
            float startAngle = 0;

            int vc = 0; // 頂点カウント

            Color wkColor = Color.Red;
            Vector3 wkNormal;

            // -------------------------------------
            // -------------------------------------
            // 頂点

            // 上 頂点 Top vertex 
            this.oVertices_頂点配列[vertCount] = new Vector3(0, 0, 1 * 半径);

            vertCount++;


            // -------------------------------------
            // 中 頂点 Middle vertices 
            alt = delta;

            for (ix = 1; ix <= totalRows_円数; ix++)
            {
                a = (float)Math.Cos(alt);

                secRad = (float)Math.Sin(alt);
                secAng = startAngle;

                for (jx = 0; jx < segment_円周_Count; ++jx)
                {
                    b = (float)Math.Cos(secAng) * secRad;
                    c = (float)Math.Sin(secAng) * secRad;

                    this.oVertices_頂点配列[vertCount] = new Vector3(b * 半径, c * 半径, a * 半径);

                    secAng += delta2;
                    vertCount++;
                }

                alt += delta;
            }


            // -------------------------------------
            // 下 頂点 Bottom vertex
            this.oVertices_頂点配列[vertCount] = new Vector3(0, 0, -1 * 半径);

            vertCount++;


            // -------------------------------------
            // -------------------------------------
            // 面 法線

            // Now make faces 	
            // 上 法線 Make top conic cap (円錐 上)
            for (ix = 1; ix <= segment_円周_Count; ++ix)
            {
                nc = (ix == segment_円周_Count) ? 1 : ix + 1;

                // ?:演算子
                /*
                if(ix == segment_円周_Count)
                {
                    nc = 1;
                }
                else
                {
                    nc = ix +1;
                }
                */

                // 面設定(配列の添え字設定)
                this.Faces_面.Set(0, ix, nc);

                NormalFromTriangle_法線設定
                    (
                    out wkNormal,
                        this.oVertices_頂点配列[this.Faces_面.A],
                        this.oVertices_頂点配列[this.Faces_面.B],
                        this.oVertices_頂点配列[this.Faces_面.C]
                    );

                wkColor = Color.Blue;

                // 極 色分け
                if ( ( ix % 2) == 0 )
                {
                    wkColor = Color.White;
                }

                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                face_面_Count++;
            }

            // 中 法線 Make midsection
            for (ix = 1; ix < totalRows_円数; ++ix)
            {
                jx = (ix - 1) * segment_円周_Count + 1;

                for (kx = 0; kx < segment_円周_Count; ++kx)
                {
                    // 四角(ポリゴン2つ分)
                    na = jx + kx;
                    nb = na + segment_円周_Count;

                    nc = (kx == (segment_円周_Count - 1)) ? jx + segment_円周_Count : nb + 1;
                    nd = (kx == (segment_円周_Count - 1)) ? jx : na + 1;

                    // -------------------------------------------------
                    // 法線を求めるための面設定 (1) a-b-c
                    this.Faces_面.Set(na, nb, nc);

                    NormalFromTriangle_法線設定
                        (
                        out wkNormal,
                            this.oVertices_頂点配列[this.Faces_面.A],
                            this.oVertices_頂点配列[this.Faces_面.B],
                            this.oVertices_頂点配列[this.Faces_面.C]
                        );

                    wkColor = Color.White;
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                    face_面_Count++;

                    // -------------------------------------------------
                    // 法線を求めるための面設定 (2) a-c-d
                    this.Faces_面.Set(na, nc, nd);

                    NormalFromTriangle_法線設定
                        (
                        out wkNormal,
                            this.oVertices_頂点配列[this.Faces_面.A],
                            this.oVertices_頂点配列[this.Faces_面.B],
                            this.oVertices_頂点配列[this.Faces_面.C]
                        );



                    // 赤道設定
                    if ((totalRows_円数 / 2) == ix)
                    {
                        wkColor = Color.Red;    // 赤道表示
                    }
                    else
                    {
                        wkColor = Color.Gray;
                    }

                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                    face_面_Count++;
                }
            }

            // 下 法線 Make bottom conic cap (円錐 下)
            na = totalVerts - 1;
            jx = (totalRows_円数 - 1) * segment_円周_Count + 1;

            for (ix = 0; ix < segment_円周_Count; ++ix)
            {
                nc = ix + jx;
                nb = (ix == segment_円周_Count - 1) ? jx : nc + 1;

                this.Faces_面.Set(na, nb, nc);

                NormalFromTriangle_法線設定
                    (
                    out wkNormal,
                        this.oVertices_頂点配列[this.Faces_面.A],
                        this.oVertices_頂点配列[this.Faces_面.B],
                        this.oVertices_頂点配列[this.Faces_面.C]
                    );

                wkColor = Color.Green;

                // 極 色分け
                if ( (ix % 2) == 1)
                {
                    wkColor = Color.White;
                }

                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                face_面_Count++;
            }


            VB_頂点バッファ = new VertexBuffer(
                graphics.GraphicsDevice,
                VertexPositionNormalColor.Size * Vertices_頂点配列.Length,
                BufferUsage.None );


            VB_頂点バッファ.SetData<VertexPositionNormalColor>(Vertices_頂点配列);


        }





        // 赤道を表示する為はに円周数を6 + (4*x)とする。
        // 6,10,14,18,22,26,30,...122...254
        // Initialize2:色変更、四角形毎に色を変える。
        // 北極:(0,1,0) 南極(0,-1,0)
        public void Initialize2(GraphicsDeviceManager graphics, int 円周, float 半径)
        {
            // cube本体
            decl_頂点宣言 = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionNormalColor.VertexElements);

            this.制限球 = new BoundingSphere(Vector3.Zero, 半径);

            int segment_円周_Count = 円周; // 円周分割数を設定

            // 円周範囲チェック
            if (円周 < 4)
            {
                segment_円周_Count = 4; // 最小円周分割数を設定
            }
            if (円周 > 1024)
            {
                segment_円周_Count = 1024; // 最大円周分割数を設定
            }


            int totalRows_円数 = segment_円周_Count / 2 - 1; //The number of rows in the sphere, not counting top and bottom verts

            int totalVerts = totalRows_円数 * segment_円周_Count + 2;
            int totalFaces = totalRows_円数 * segment_円周_Count * 2;   // 面(ポリゴン)数                       

            Vertices_頂点配列 = new VertexPositionNormalColor[totalFaces * 3]; // 頂点数 = ポリゴン数 * 3


            this.oVertices_頂点配列 = new Vector3[totalVerts];

            int ix, jx, kx;		// Index variables
            int na, nb, nc, nd; // Temporary integers

            int vertCount = 0;	// Counters for the number of vertices

            float a, b, c;					//Temporary floats
            float alt, secRad, secAng;		//Altitude, radial secant, angular secant

            float delta = 2.0f * MathHelper.Pi / segment_円周_Count;
            float delta2 = delta;
            float startAngle = 0;

            int vc = 0; // 頂点カウント

            Color wkColor = Color.Red;
            Vector3 wkNormal;

            Color 色1 = Color.White;
            Color 色2 = Color.Gray;

            // -------------------------------------
            // -------------------------------------
            // 頂点

            // 上 頂点 Top vertex 
            this.oVertices_頂点配列[vertCount] = new Vector3(0, 1 * 半径, 0);

            vertCount++;


            // -------------------------------------
            // 中 頂点 Middle vertices 
            alt = delta;

            for (ix = 1; ix <= totalRows_円数; ix++)
            {
                a = (float)Math.Cos(alt);

                secRad = (float)Math.Sin(alt);
                secAng = startAngle;

                for (jx = 0; jx < segment_円周_Count; ++jx)
                {
                    b = (float)Math.Cos(secAng) * secRad;
                    c = (float)Math.Sin(secAng) * secRad;

                    this.oVertices_頂点配列[vertCount] = new Vector3(b * 半径, a * 半径, c * 半径);

                    secAng += delta2;
                    vertCount++;
                }

                alt += delta;
            }


            // -------------------------------------
            // 下 頂点 Bottom vertex
            this.oVertices_頂点配列[vertCount] = new Vector3(0, -1 * 半径, 0);

            vertCount++;


            // -------------------------------------
            // -------------------------------------
            // 面 法線

            // Now make faces 	
            // 上 法線 Make top conic cap (円錐 上)
            for (ix = 1; ix <= segment_円周_Count; ++ix)
            {
                nc = (ix == segment_円周_Count) ? 1 : ix + 1;

                // ?:演算子
                /*
                if(ix == segment_円周_Count)
                {
                    nc = 1;
                }
                else
                {
                    nc = ix +1;
                }
                */

                // 面設定(配列の添え字設定)
                this.Faces_面.Set(0, ix, nc);

                NormalFromTriangle_法線設定
                    (
                    out wkNormal,
                        this.oVertices_頂点配列[this.Faces_面.A],
                        this.oVertices_頂点配列[this.Faces_面.B],
                        this.oVertices_頂点配列[this.Faces_面.C]
                    );

                wkColor = Color.Green;

                // 中心が分かるように、1本のみ色を変る。
                if(1 == ix)
                {
                    wkColor = 色1;
                }

                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                face_面_Count++;
            }



            // ==================================================================================
            // 中 法線 Make midsection
            for (ix = 1; ix < totalRows_円数; ++ix)
            {
                jx = (ix - 1) * segment_円周_Count + 1;

                for (kx = 0; kx < segment_円周_Count; ++kx)
                {

                    // ------------------------------------
                    // 色設定
                    if (ix % 2 == 1)
                    {
                        if (kx % 2 == 1)
                        {
                            wkColor = 色1;

                            // 赤道設定
                            if ((totalRows_円数 / 2) == ix)
                            {
                                wkColor = Color.Red;    // 赤道表示
                            }
                        }
                        else
                        {
                            wkColor = 色2;
                        }
                    }
                    else
                    {
                        if (kx % 2 == 1)
                        {
                            wkColor = 色2;

                            // 赤道設定
                            if ((totalRows_円数 / 2) == ix)
                            {
                                wkColor = Color.Red;    // 赤道表示
                            }
                        }
                        else
                        {
                            wkColor = 色1;
                        }
                    }




                    // 四角(ポリゴン2つ分)
                    na = jx + kx;
                    nb = na + segment_円周_Count;

                    nc = (kx == (segment_円周_Count - 1)) ? jx + segment_円周_Count : nb + 1;
                    nd = (kx == (segment_円周_Count - 1)) ? jx : na + 1;

                    // -------------------------------------------------
                    // 法線を求めるための面設定 (1) a-b-c
                    this.Faces_面.Set(na, nb, nc);

                    NormalFromTriangle_法線設定
                        (
                        out wkNormal,
                            this.oVertices_頂点配列[this.Faces_面.A],
                            this.oVertices_頂点配列[this.Faces_面.B],
                            this.oVertices_頂点配列[this.Faces_面.C]
                        );

                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                    face_面_Count++;

                    // -------------------------------------------------
                    // 法線を求めるための面設定 (2) a-c-d
                    this.Faces_面.Set(na, nc, nd);

                    NormalFromTriangle_法線設定
                        (
                        out wkNormal,
                            this.oVertices_頂点配列[this.Faces_面.A],
                            this.oVertices_頂点配列[this.Faces_面.B],
                            this.oVertices_頂点配列[this.Faces_面.C]
                        );
                    
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                    Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                    face_面_Count++;
                }
            }


            
            
            
            // 下 法線 Make bottom conic cap (円錐 下)
            na = totalVerts - 1;
            jx = (totalRows_円数 - 1) * segment_円周_Count + 1;

            for (ix = 0; ix < segment_円周_Count; ++ix)
            {
                nc = ix + jx;
                nb = (ix == segment_円周_Count - 1) ? jx : nc + 1;

                this.Faces_面.Set(na, nb, nc);

                NormalFromTriangle_法線設定
                    (
                    out wkNormal,
                        this.oVertices_頂点配列[this.Faces_面.A],
                        this.oVertices_頂点配列[this.Faces_面.B],
                        this.oVertices_頂点配列[this.Faces_面.C]
                    );

                wkColor = Color.Blue;

                // 中心が分かるように、1本のみ色を変る。
                if (1 == ix)
                {
                    wkColor = 色2;
                }

                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.A], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.B], wkNormal, wkColor);
                Vertices_頂点配列[vc++] = new VertexPositionNormalColor(this.oVertices_頂点配列[this.Faces_面.C], wkNormal, wkColor);

                face_面_Count++;
            }


            VB_頂点バッファ = new VertexBuffer(
                graphics.GraphicsDevice,
                VertexPositionNormalColor.Size * Vertices_頂点配列.Length,
                BufferUsage.None);

            VB_頂点バッファ.SetData<VertexPositionNormalColor>(Vertices_頂点配列);
        }










        // --------------------------------------
        public void Draw(   GraphicsDeviceManager graphics,
                            BasicEffect effect,     // ライティング+テクスチャ 
                            Matrix view,
                            Matrix proj)
        {
/*
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);              // 回転

            // ワールド * ビュー * 射影行列
            m = m * view * projection;

            effect.Projection = m;
*/

            effect.World = this.単位行列 * Matrix.CreateTranslation(this.pos);
            effect.View = view;
            effect.Projection = proj;


            effect.TextureEnabled = false;
            effect.VertexColorEnabled = true;  // 追加!! このエフェクトの頂点色を使用できるようします。 

            graphics.GraphicsDevice.VertexDeclaration = decl_頂点宣言;
            graphics.GraphicsDevice.Vertices[0].SetSource(VB_頂点バッファ, 0, VertexPositionNormalColor.Size);

            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;  // カリング ポリゴン両面表示


            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawPrimitives(
                    PrimitiveType.TriangleList,
                    0,
                    this.face_面_Count  // 表示ポリゴン数
                );
                /*
                primitiveType
                    レンダリングするプリミティブの型について説明します。
                startVertex
                    読み込む最初の頂点のインデックス。 先頭 startVertex, 頂点バッファの外読み取りは、正しい頂点数。
                primitiveCount
                    レンダリングするプリミティブの数です。 確認できるプリミティブの最大数をチェックします。 MaxPrimitiveCount. します primitiveCount プリミティブ タイプによって決まるとプリミティブの数です。 場合、線リストは 2 つの頂点はそれぞれプリミティブです。 場合、トライアングル リストは 3 つの頂点はそれぞれプリミティブです。
                */

                pass.End();
            }
            effect.End();

            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;  // CullMode元に

        }








        // ========================
        // ========================
        // ========================



        public Vector3[] oVertices_頂点配列;

        public Face Faces_面;

        public int face_面_Count = 0;  // 面(ポリゴン)の数Counters for the number of faces 


        /// <summary>
        /// 法線設定 
        /// </summary>
        /// <param name="normal"></param>
        /// <param name="p0"></param>
        /// <param name="p1"></param>
        /// <param name="p2"></param>
        private static void NormalFromTriangle_法線設定(out Vector3 normal, Vector3 p0, Vector3 p1, Vector3 p2)
        {
            // Computes a normal from three triangle vertices
            Vector3 v0 = p1 - p0;
            Vector3 v1 = p2 - p0;

            normal = Vector3.Cross(v0, v1);

            normal.Normalize();
        }




        // --------------------------------------
        /// <summary>
        /// レイとBoundingSphereの交差判定
        /// 戻り値 : 交差無し=null / 交差するまでの距離=交差有り
        /// BoundingSphereを前もって設定する。
        /// </summary>
        /// <param name="ray"></param>
        /// <returns></returns>
        virtual public float? BoundingSphereレイ判定(Ray ray)
        {
            /*
            Matrix M11 - M44
            11 12 13 14 -X
            21 22 23 24 -Y
            31 32 33 34 -Z
            41 42 43 44
            Matrix m = Matrix.CreateTranslation( new Vector3(777,888,999) ); // M31,M32,M33に入る
             */

            Matrix pos = Matrix.CreateTranslation(ray.Position);
            Matrix dir = Matrix.CreateTranslation(ray.Direction);
            Matrix inv = Matrix.Invert(this.単位行列);

            pos *= Matrix.CreateTranslation(-this.pos) * inv;
            dir *= inv;

            Ray wkRay = new Ray(pos.Translation, dir.Translation);// pos,方向

            Nullable<float> result = wkRay.Intersects(this.制限球);

            return result;  // 交差有りの場合:交差するまでの距離
        }
        // --------------------------------------








    }

}
