﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;

namespace ccc4
{

    // Matrixによるカメラ
    // カメラ位置 : m_単位行列.Backward * m_len
    // UPベクトル : m_単位行列.Up
    class カメラ
    {
        Matrix m_単位行列 = Matrix.Identity;    // カメラ角度

        Vector3 m_pos = Vector3.Zero;           // カメラ座標

        float m_len = 3200f;                     // カメラ - 参照点間の距離

        // カメラ各軸の角度 : x-y軸の回転 (2軸) に使用
        float m_angX = 0f;  
        float m_angY = 0f;
        float m_angZ = 0f;

        public カメラ()
        {
        }

        public void 初期化()
        {
            m_単位行列 = Matrix.Identity;     // カメラ角度

            m_pos = Vector3.Zero;             // カメラ座標

            m_len = 3200;                     // カメラ - 参照点間の距離

            // カメラ各軸の角度 : x-y軸の回転 (2軸) に使用(ラジアン)
            m_angX = 0f;
            m_angY = 0f;
            m_angZ = 0f;

        }


        // アクセサ ----------------------------------------------------------------------
        public Matrix 単位行列
        {
            get { return m_単位行列; }
            set { m_単位行列 = value; }
        }

        public Vector3 pos
        {
            get { return this.m_単位行列.Backward * this.m_len; }
        }

        public float len
        {
            get { return m_len; }
            set { m_len = value; }
        }

        public float angX
        {
            get { return m_angX; }
            set { m_angX = value; }
        }

        public float angY
        {
            get { return m_angY; }
            set { m_angY = value; }
        }

        public float angZ
        {
            get { return m_angZ; }
            set { m_angZ = value; }
        }


        // ====================================================================
        // x軸の回転
        // d:回転角(ラジアン)
        public void RotateX(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateRotationX(d);

            c単位行列正規化();	// 一定間隔毎にカメラの単位行列の正規化
        }

        // y軸の回転
        public void RotateY(float d)
        {
            if (0f == d)
            {
                return;
            }
            m_単位行列 *= Matrix.CreateRotationY(d);

            c単位行列正規化();	// 一定間隔毎にカメラの単位行列の正規化

        }

        // z軸の回転
        public void RotateZ(float d)
        {
            if (0f == d)
            {
                return;
            }
            m_単位行列 *= Matrix.CreateRotationZ(d);

            c単位行列正規化();	// 一定間隔毎にカメラの単位行列の正規化
        }

        // x-y軸の回転 (2軸)
        // 参照点を中心にカメラが球上を移動するときに使用
        public void RotateXY(float x,float y)
        {
            /*
            if (0f == x && 0f == y)
            {
                return;
            }
            */

            m_angX += x;
            m_angY += y;

            m_単位行列  = Matrix.CreateRotationX(m_angX);
            m_単位行列 *= Matrix.CreateRotationY(m_angY);
        }




        // x-y軸の回転 (2軸) 
        // 参照点を中心にカメラが球上を移動するときに使用
        // 上下方向(x軸回転)の角度制限付き
        // ワールド表示用。角度が0の場合終了
        public void RotateXYa(float x, float y, float 制限角)
        {
            if (0f == x && 0f == y)
            {
                return;
            }

            m_angX += x;
            m_angY += y;

            // 上下カメラ角度制限
            if (m_angX >  制限角) { m_angX =  制限角; }
            if (m_angX < -制限角) { m_angX = -制限角; }

            m_単位行列 =  Matrix.CreateRotationX(m_angX);
            m_単位行列 *= Matrix.CreateRotationY(m_angY);
        }










        // x-y軸の回転 (2軸) 
        // 参照点を中心にカメラが球上を移動するときに使用
        // 上下方向(x軸回転)の角度制限付き
        // ユニット表示用。次にユニットの単位行列を掛けるので、無条件に実行
        public void RotateXYa_unit(float x, float y, float 制限角)
        {
            m_angX += x;
            m_angY += y;

            // 上下カメラ角度制限
            if (m_angX >  制限角) { m_angX =  制限角; }
            if (m_angX < -制限角) { m_angX = -制限角; }

            m_単位行列 =  Matrix.CreateRotationX(m_angX);
            m_単位行列 *= Matrix.CreateRotationY(m_angY);
        }





        // ====================================================================
        // 単位行列の各軸を中心とした回転

        // ピッチ(x軸の回転)
        // d:回転角(ラジアン)
        public void RotateX_Pitch(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Right, d);

            c単位行列正規化();	// 一定間隔毎にカメラの単位行列の正規化
        }


        // ヨー(y軸の回転)
        public void RotateY_Yaw(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Up, d);

            c単位行列正規化();	// 一定間隔毎にカメラの単位行列の正規化
        }


        // ロール(z軸の回転)
        public void RotateZ_Roll(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Backward, d);

            c単位行列正規化();	// 一定間隔毎にカメラの単位行列の正規化
        }



        // ====================================================================
        // ビュー変換行列 作成
        public Matrix view(Vector3 参照点)
        {
            // ビュー変換行列
            return Matrix.CreateLookAt( this.m_単位行列.Backward * this.m_len,  // カメラ位置
                                        参照点,                                 // 参照点
                                        this.m_単位行列.Up );                   // UPベクトル
        }



        public int c単位行列正規化カウンタ = 0;

        /// <summary>
        /// 一定間隔毎にカメラの単位行列の正規化
        /// </summary>
        private void c単位行列正規化()
        {
            c単位行列正規化カウンタ++;
            if (2048 < c単位行列正規化カウンタ)
            {
                float angX, angY, angZ;

                Matrix wkM, wkMA;

                wkM = this.単位行列;

                // AbsX > AbsZ:z軸角度から求める。
                // AbsX < AbsZ:x軸角度から求める。

                if (Math.Abs(this.単位行列.Up.X) < Math.Abs(this.単位行列.Up.Z))
                {
                    // --------------------------------------------------------------------------------
                    angX = (float)Math.Atan2(this.単位行列.Up.Z, this.単位行列.Up.Y);
                    wkM *= Matrix.CreateRotationX(-angX);   // x 回転

                    // --------------------------------------------------------------------------------
                    angZ = (float)Math.Atan2(wkM.Up.X, wkM.Up.Y);
                    wkM *= Matrix.CreateRotationZ(angZ);    // z 回転

                    // --------------------------------------------------------------------------------
                    angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                    // 新しい単位行列
                    wkMA = Matrix.CreateRotationY(angY);
                    wkMA *= Matrix.CreateRotationZ(-angZ);
                    wkMA *= Matrix.CreateRotationX(angX);
                }
                else
                {
                    // --------------------------------------------------------------------------------
                    angZ = (float)Math.Atan2(this.単位行列.Up.X, this.単位行列.Up.Y);
                    wkM *= Matrix.CreateRotationZ(angZ);   // z 回転

                    // --------------------------------------------------------------------------------
                    angX = (float)Math.Atan2(wkM.Up.Z, wkM.Up.Y);
                    wkM *= Matrix.CreateRotationX(-angX);    // x 回転

                    // --------------------------------------------------------------------------------
                    angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                    // 新しい単位行列
                    wkMA = Matrix.CreateRotationY(angY);
                    wkMA *= Matrix.CreateRotationX(angX);
                    wkMA *= Matrix.CreateRotationZ(-angZ);
                }
                // ===============================================================
                // ===============================================================

                this.単位行列 = wkMA;

                c単位行列正規化カウンタ = 0;
            }
        }







    
    }
}
