﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;


namespace ccc4
{
    class 軌跡
    {
        VertexDeclaration decl = null;

        // 線によるunit軌跡
        VertexPositionColor[]   vpc軌跡 = null;
        VertexBuffer            vb軌跡; // Vertex buffer to define a triangle.

        VertexPositionColor[]   vpc接続 = null;
        VertexBuffer            vb接続; // Vertex buffer to define a triangle.


        Color   m_col軌跡 = Color.Red;

//      private int vb軌跡size = 5500;
//      private int m_軌跡長   = 60 * 90;  // 描画する軌跡長(5400)vb軌跡sizeより小さいこと。

        // 軌跡
        private int vb軌跡size  = 4096;  // 配列長     

        public Color col軌跡
        {
            set { m_col軌跡 = value; }
        }


        // --------------------------------------
        public 軌跡()
        {
        }

        public void Initialize(GraphicsDeviceManager graphics )
        {
            decl = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            // ---------------------------
            // 線による軌跡
            vb軌跡  = new VertexBuffer(
                graphics.GraphicsDevice, 
                typeof(VertexPositionColor),
                vb軌跡size,
                BufferUsage.None );

            vpc軌跡 = new VertexPositionColor[vb軌跡size];

            // ---------------------------
            // 配列の終点-始点間を接続
            vb接続 = new VertexBuffer(
                graphics.GraphicsDevice,
                typeof(VertexPositionColor),
                2,
                BufferUsage.None );

            vpc接続 = new VertexPositionColor[2];
        }


        //	軌跡設定
        public void 設定( Vector3 pos, int GameCount)
        {
            // -------------------------------------------
            // 軌跡 設定
            int i = GameCount % vb軌跡size;

            vpc軌跡[i].Position = pos;
            vpc軌跡[i].Color = this.m_col軌跡;

            vb軌跡.SetData<VertexPositionColor>(vpc軌跡);


            // ------------
            // 配列の終点-始点間を接続 (そのままだと軌跡が引かれないので)
            // 接続 始点 (配列の終わり)
            if (i == (vb軌跡size - 1))
            {
                vpc接続[0].Position = pos;
                vpc接続[0].Color = this.m_col軌跡;
                vb接続.SetData<VertexPositionColor>(vpc接続);
            }

            // 接続 終点 (配列の始め)
            if (i == 0 )
            {
                vpc接続[1].Position = pos;
                vpc接続[1].Color = this.m_col軌跡;
                vb接続.SetData<VertexPositionColor>(vpc接続);
            }

        
        }


        // 指定座標で軌跡を初期化
        public void 初期化(Vector3 pos)
        {
            if (null == vb軌跡)
            {
                return;
            }

            // -------------------------------------------
            // 軌跡 設定
            // リソースのロック (CPUによる記憶領域へのアクセスを許可)
            for (int i = 0; i < vpc軌跡.Length; i++)
            {
                vpc軌跡[i].Position = pos;
                vpc軌跡[i].Color = this.m_col軌跡;
            }

            vb軌跡.SetData<VertexPositionColor>(vpc軌跡);


            // ------------
            // 接続 始点
            vpc接続[0].Position = pos;
            vpc接続[0].Color = this.m_col軌跡;

            // 接続 終点
            vpc接続[1].Position = pos;
            vpc接続[1].Color = this.m_col軌跡;

            vb接続.SetData<VertexPositionColor>(vpc接続);
        }





        // unit軌跡表示
        public void Draw(   GraphicsDeviceManager graphics,
                            Effect  effect,
                            Matrix  view,
                            Matrix  proj,
                            int     GameCount,
                            int     軌跡長 )
        {
            // 軌跡長 : 描画する軌跡長(注意! vb軌跡sizeより小さいこと。チェック省略)

            // ワールド * ビュー * 射影行列
            effect.Parameters["WorldViewProj"].SetValue(view * proj);

            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            // --------------

            int startV = 0;
            int プリミティブ数 = 0;

            int i = GameCount % vb軌跡size;

            /*
            i = 0 % 3;    // 0
            i = 1 % 3;    // 1 
            i = 2 % 3;    // 2
            i = 3 % 3;    // 0
            i = 4 % 3;    // 1

        int vb軌跡size = 3;
        int m_軌跡長   = 2;     // 描画する軌跡長
             */

            graphics.GraphicsDevice.Vertices[0].SetSource(this.vb軌跡, 0, VertexPositionColor.SizeInBytes);

            // iが軌跡長以下の場合、描画する軌跡が配列内で2つに分かれている
            if (i <= 軌跡長)
            {
                // 軌跡が配列内、戦後2つに格納
                // 0 ------------------------- 511 ->
                // oo........................ooooo
                // aa                        bbbbb
                // DrawPrimitives()を2回実行

                // startV:        1 = (3-1)-(2-1)
                // プリミティブ数:1 = 2 - 1
                startV         = (vb軌跡size - 1) - (軌跡長 - i); // ロードする最初の頂点のインデックス
                プリミティブ数 = 軌跡長 - i;			            // レンダリングするプリミティブ数

                if (0 < プリミティブ数)
                {
                    effect.Begin();
                    foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                    {
                        pass.Begin();

                        // プリミティブのシーケンスをレンダリング bbbb部分
                        graphics.GraphicsDevice.DrawPrimitives(
                            PrimitiveType.LineStrip,	// レンダリングするプリミティブの種類
                            startV,						// ロードする最初の頂点のインデックス
                            プリミティブ数);			// レンダリングするプリミティブ数

                        pass.End();
                    }
                    effect.End();
                }

                // aa部分
                // 配列の初めから途中までを描画
                startV = 0;
                プリミティブ数 = i - 1;
            }
            else
            {
                // 配列の途中から途中までを描画
                startV = i - (軌跡長 + 1);
                プリミティブ数 = 軌跡長;
            }


            if (0 < プリミティブ数)
            {
                effect.Begin();
                foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                {
                    pass.Begin();

                    // プリミティブのシーケンスをレンダリング aa部分
                    graphics.GraphicsDevice.DrawPrimitives(
                        PrimitiveType.LineStrip,	// レンダリングするプリミティブの種類
                        startV,						// ロードする最初の頂点のインデックス
                        プリミティブ数);			// レンダリングするプリミティブ数

                    pass.End();
                }
                effect.End();
            }

            // 配列の終点-始点間を接続 (そのままだと軌跡が引かれないので)
            if (i < 軌跡長)
            {
                graphics.GraphicsDevice.Vertices[0].SetSource(this.vb接続, 0, VertexPositionColor.SizeInBytes);

                effect.Begin();
                foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                {
                    pass.Begin();

                    graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, 1);

                    pass.End();
                }
                effect.End();
            }

        
        }

    
    
    
    }
}
