﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Xml;

namespace ccc4
{
    public partial class 地形ダイアログ : Form
    {
        public 地形ダイアログ( Game1 Game1 )
        {
            InitializeComponent();

            this.m_Game1 = Game1;

            // ボリューム設定
            this.trackBar1.Value = (int)(this.m_Game1.musicVolume * 20);

            // 操作上下リバース 飛行モード ピッチ + カメラ上下反転 
            this.checkBox1.Checked = this.m_Game1.b上下リバース;
        }


        private Game1       m_Game1;

        private XmlDocument m_XmlDoc   = new XmlDocument();
        private string      m_FileName = null;     // unit定義ファイル名 


       
        // 地形 開く
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                // フォルダ (ディレクトリ) が存在しているかどうか確認する
                if (!System.IO.Directory.Exists(this.m_Game1.InitialDirectory))
                {
                    // ディレクトリが存在しません
                    this.m_Game1.InitialDirectory = System.Windows.Forms.Application.StartupPath;
                }

                // 起動ディレクトリ設定
                openFileDialog1.InitialDirectory = this.m_Game1.InitialDirectory;

                // [ファイルを開く] ダイアログの表示
                if (System.Windows.Forms.DialogResult.OK == openFileDialog1.ShowDialog())
                {
                    // 起動ディレクトリ保存
                    this.m_Game1.InitialDirectory = System.IO.Path.GetDirectoryName(this.openFileDialog1.FileName);


                    this.m_FileName = openFileDialog1.FileName;
                    this.m_Game1.地形XMLアニメload(this.m_FileName);

/*
#if(DEBUG)
                    // debug 確認用
                    this.m_XmlDoc.Load(openFileDialog1.FileName);   // XML文書の読み込み
                    this.m_XmlDoc.Normalize();
                    this.m_XmlDoc.Save(@"デバッグ出力.xml");
#endif
*/
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message,	// テキスト
                    exc.GetType().ToString(),	// キャプション
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }

        }


        // 地形 保存
        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                // フォルダ (ディレクトリ) が存在しているかどうか確認する
                if (!System.IO.Directory.Exists(this.m_Game1.InitialDirectory))
                {
                    // ディレクトリが存在しません
                    this.m_Game1.InitialDirectory = System.Windows.Forms.Application.StartupPath;
                }

                // 起動ディレクトリ設定
                saveFileDialog1.InitialDirectory = this.m_Game1.InitialDirectory;

                // [名前を付けて保存] ダイアログの表示
                if (System.Windows.Forms.DialogResult.OK == this.saveFileDialog1.ShowDialog())
                {
                    // 起動ディレクトリ保存
                    this.m_Game1.InitialDirectory = System.IO.Path.GetDirectoryName(this.saveFileDialog1.FileName);

                    // -------------------------------------------
                    // 地形クラスの内容を.xmlに保存
                    /*
                    http://dobon.net/vb/dotnet/file/xmlserializer.html
                    XMLファイルに書き込まれる要素は、パブリックメンバであるフィールドとプロパティだけです。

                    さらに、読み込み、あるいは書き込みしか許可されていないプロパティは、
                    パブリックメンバであっても書き込まれません。

                    また、シリアル化するオブジェクトのクラスには、
                    規定のコンストラクタ（引数を持たないコンストラクタ）が必要です。
                    */
                    // ----------------------------

                    //保存先のファイル名 (exeのフルパスを取得し、そのディレクトリ名を得る)
                    string fileName = this.saveFileDialog1.FileName;

                    // ＜XMLファイルに書き込む＞
                    // XmlSerializerオブジェクトを作成
                    // 書き込むオブジェクトの型を指定する
                    System.Xml.Serialization.XmlSerializer serializer1 = new System.Xml.Serialization.XmlSerializer(typeof(地形));

                    // ファイルを開く
                    System.IO.FileStream fs1 = new System.IO.FileStream(fileName, System.IO.FileMode.Create);


                    // 保存対象外項目を初期化
                    this.m_Game1.m_地形.musicVolume = 0f;
                    this.m_Game1.m_地形.InitialDirectory = "";
                    this.m_Game1.m_地形.b上下リバース = true;

                    // シリアル化し、XMLファイルに保存する
                    serializer1.Serialize( fs1, this.m_Game1.m_地形 );

                    // 閉じる
                    fs1.Close();
                    // -------------------------------------------
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message,	// テキスト
                    exc.GetType().ToString(),	// キャプション
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }
        }



        // 地形 初期化 aaa
        private void button3_Click(object sender, EventArgs e)
        {
            if (System.Windows.Forms.DialogResult.Cancel ==
                System.Windows.Forms.MessageBox.Show(
                "現在の地形を破棄して、初期化します。\nよろしいですか?",
                "新規作成",    // キャプション
                System.Windows.Forms.MessageBoxButtons.OKCancel,
                System.Windows.Forms.MessageBoxIcon.Question))
            {
                return; // キャンセルの場合終了
            }

            // ============================================
            // 地形.ビル 初期化 
//          this.m_Game1.地形初期化();
            this.m_Game1.地形アニメ初期化();
        }



        // キャンセル
        private void button4_Click(object sender, EventArgs e)
        {

        }



        // サウンド ボリューム 設定 トラックバー
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            // 0 ～ 20 => 0.0 ～ 1.0 に変換
            this.m_Game1.ボリューム設定( (float)this.trackBar1.Value / 20f );
        }


        // 上下リバース チェックボックス 
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            this.m_Game1.m_地形.b上下リバース = this.checkBox1.Checked;
            this.m_Game1.b上下リバース = this.checkBox1.Checked;
        }




    }
}