﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;


namespace ccc4
{
    // 複数のBox2でビルを構成
    // テクスチャを
    // basicEffect.DiffuseColor = new Vector3(0.8f, 0.8f, 0.8f);

    public class ビル : 基本Model
    {
        [System.Xml.Serialization.XmlIgnoreAttribute]
        public Box2 外壁 = new Box2(); // 半透明表示

        [System.Xml.Serialization.XmlIgnoreAttribute]
        public cubeS    床 = new cubeS();  // 複数の床 + 天井


        [System.Xml.Serialization.XmlIgnoreAttribute]
        public cube 柱;  // 4本の柱

        public float m_上w;
        public float m_上d;
        public float m_下w;
        public float m_下d;
        public float m_h;
        public int   m_階数 = 0;

        [System.Xml.Serialization.XmlIgnoreAttribute]
        public float length;	// 視点からの距離 (sort対象)

        public int m_外壁テクスチャNo = 1;
        public int m_床テクスチャNo   = 1;

        Vector3[] 柱pos = new Vector3[4];

        float 床高 = 0; // 床と床の間隔
        float 床厚 = 0.2f; // 床の厚さ ttt

        Box2 m_表示BoundingBox = new Box2(); // 選択時表示バウンディングボックス


        float def_h = 4f;   // 1階分の高さ

        float def_max = 128f; // 幅、奥行きの最大値
        float def_min =   4f; // 幅、奥行きの最小値

        float m_maxWD = 0f;  // 幅、奥行きの最大値、ユニットが球とビルとの隙間に居る場合の判定に使用

        // --------------------------------------
        public ビル()
        {
        }


        // アクセサ ----------------------------------------------------------------------

        // 選択時表示バウンディングボックス
        public Box2 表示BoundingBox { 
            get { return m_表示BoundingBox; }
        }

        /// <summary>
        /// 幅、奥行きの最大値、ユニットが球とビルとの隙間に居る場合の判定に使用
        /// </summary>
        public float maxWD
        {
            get { return m_maxWD; }
        }


        // 床pos、単位行列設定 : (Initialize時、ビルの移動時)
        /// <summary>
        /// 床のposと単位行列を再設定( 順序注意 ビルのposと行列設定後に設定 )
        /// </summary>
        public void 床pos設定()
        {
            this.床.pos      = this.pos;
            this.床.単位行列 = this.単位行列;
        }




        
        
        

        
        // --------------------------------------
        // 上側の幅w、奥行きd, 下側の幅w、奥行きd, 高さh, 階数 を指定
        public void Initialize(GraphicsDeviceManager graphics,
                            bool    chk,    // 階数による高さチェックと設定の有無
                            
                            float   上w,
                            float   上d,
                            float   下w,
                            float   下d,

                            float   h,
                            
                            int     階数 )
        {
            // ビルの1階分の高さ = 最低:4

            this.m_上w  = 上w;
            this.m_上d  = 上d;
            this.m_下w  = 下w;
            this.m_下d  = 下d;
            this.m_h    = h;

            this.m_階数 = 階数;


            // --------------------------------------------------------
            // 最小値チェック
            if (this.m_上w < def_min)
            {
                this.m_上w = def_min;
            }
            if (this.m_上d < def_min)
            {
                this.m_上d = def_min;
            }
            if (this.m_下w < def_min)
            {
                this.m_下w = def_min;
            }
            if (this.m_下d < def_min)
            {
                this.m_下d = def_min;
            }

            if (this.m_h < def_h)
            {
                this.m_h = def_h;
            }

            // ビルの1階分の高さ = 最低:4
            if (this.m_h < (this.m_階数 * def_h))
            {
                this.m_h = (this.m_階数 * def_h);
            }

            // --------------------------------------------------------
            // 最大値チェック
            if (this.m_上w > def_max)
            {
                this.m_上w = def_max;
            }
            if (this.m_上d > def_max)
            {
                this.m_上d = def_max;
            }
            if (this.m_下w > def_max)
            {
                this.m_下w = def_max;
            }
            if (this.m_下d > def_max)
            {
                this.m_下d = def_max;
            }

            if (this.m_h > (def_h * 100))
            {
                this.m_h = (def_h * 100);
            }

            // --------------------------------------------------------
            // 階数チェック chk == false の場合,階数=1に設定  最大100階
            if (this.m_階数 < 1 || !chk)
            {
                this.m_階数 = 1;
            }
            else if (this.m_階数 > 100)
            {
                this.m_階数 = 100;
            }

            this.外壁.Initialize(graphics, this.m_上w, this.m_上d, this.m_下w, this.m_下d, this.m_h, this.m_階数);

            // --------
            // ビル選択時 外壁 のバウンディングボックス表示
            Vector3 v3max = this.外壁.BoundingBoxA.Max;
            Vector3 v3min = this.外壁.BoundingBoxA.Min;

            m_表示BoundingBox.Initialize(graphics,
                            v3max.X * 2, v3max.Z * 2,
                            v3max.X * 2, v3max.Z * 2,
                            this.m_h, this.m_階数);

            // -------------------------------------
            // 幅、奥行きの最大値、ユニットが球とビルとの隙間に居る場合の判定に使用
            this.m_maxWD = v3max.X * 2;

            if (this.m_maxWD < (v3max.Z * 2))
            {
                this.m_maxWD = v3max.Z * 2;
            }

            // -------------------------------------
            this.床.Initialize(graphics);

            this.床高 = this.m_h / this.m_階数;

            // ------------------------------------------------------------

            float 増分床高 = this.床高;

            float fw = (this.m_上w - this.m_下w) / this.m_h;
            float fd = (this.m_上d - this.m_下d) / this.m_h;

            for (int i = 0; i < this.m_階数; i++)
            {
                float 増分w = m_下w + (fw * 増分床高);
                float 増分d = m_下d + (fd * 増分床高);

                if (増分w < def_min) { 増分w = def_min; }
                if (増分d < def_min) { 増分d = def_min; }

                床.add(graphics, 増分w, 増分d, 床厚, 増分床高);

                増分床高 += this.床高;
            }

            // ================================================================
            // 柱の座標 
            float 柱径 = 0.4f;
            float 柱径2 = 柱径 / 2;

            柱 = new cube();   // 4本の柱(同一のBox2を座標を変えて4回描画)
            柱.Initialize(graphics, false, 柱径, 柱径, this.m_h + (def_h * 2)); // 太さ1の柱。(def_h * 2):柱を下に延ばす。

            float 上w2 = (this.m_上w / 2) - 柱径2;
            float 上d2 = (this.m_上d / 2) - 柱径2;
            float 下w2 = (this.m_下w / 2) - 柱径2;
            float 下d2 = (this.m_下d / 2) - 柱径2;

            // 外壁と重ならないように少し小さくする。
            上w2 *= 0.99f;
            上d2 *= 0.99f;
            下w2 *= 0.99f;
            下d2 *= 0.99f;


            // 上下の幅、奥行きの小さいほうに柱を置く
            for (int i = 0; i < 4; i++)
            {
                柱pos[i] = Vector3.Zero;
                柱pos[i].Y = -(def_h * 2); // 柱を下に延ばす。
            }

            if (上w > 下w)
            {
                柱pos[0].X = -下w2;
                柱pos[1].X =  下w2;
                柱pos[2].X = -下w2;
                柱pos[3].X =  下w2;
            }
            else
            {
                柱pos[0].X = -上w2;
                柱pos[1].X =  上w2;
                柱pos[2].X = -上w2;
                柱pos[3].X =  上w2;
            }

            if (上d > 下d)
            {
                柱pos[0].Z =  下d2;
                柱pos[1].Z =  下d2;
                柱pos[2].Z = -下d2;
                柱pos[3].Z = -下d2;
            }
            else
            {
                柱pos[0].Z =  上d2;
                柱pos[1].Z =  上d2;
                柱pos[2].Z = -上d2;
                柱pos[3].Z = -上d2;
            }

        } // Initialize







        
        
        
        // --------------------------------------
        // 外壁を半透明表示+床と柱を表示
        public void Draw詳細(   GraphicsDeviceManager graphics,
                                BasicEffect effect,     // ライティング+テクスチャ 
                                Matrix view,
                                Matrix proj )
        {
            // 柱の角度設定
            柱.単位行列 = this.単位行列;
            for (int i = 0; i < 4; i++)
            {
                // 柱のpos設定
                Matrix m = Matrix.CreateTranslation(柱pos[i]) * this.単位行列;
                柱.pos = this.pos + m.Translation;

                柱.Drawシンプル(graphics, effect, view, proj);
            }

            床.Draw(graphics, effect, view, proj);

            // 表示切替
            // カメラとの距離が遠い場合、外壁のみ表示
            // 近い場合、外壁を半透明、柱と床を表示

            // 半透明は最後に描画
            this.外壁.pos = this.pos;
            this.外壁.単位行列 = this.単位行列;
            this.外壁.Drawシンプルa(graphics, effect, view, proj);
        }


        // --------------------------------------
        // ビル選択時にバウンディングボックスを表示
        public void DrawBoundingBox(    GraphicsDeviceManager graphics,
                                        BasicEffect effect,     // ライティング+テクスチャ 
                                        Matrix view,
                                        Matrix proj)
        {
            m_表示BoundingBox.pos = this.pos;
            m_表示BoundingBox.単位行列 = this.単位行列;

            m_表示BoundingBox.Drawシンプルa(graphics, effect, view, proj);
        }



////////////////////////////////////
////////////////////////////////////
/*
            float w2 = 0;
            float d2 = 0;
            if (上w2 > 下w2)
            {
                w2 = 上w2;
            }
            else
            {
                w2 = 下w2;
            }
            if (上d2 > 下d2)
            {
                d2 = 上d2;
            }
            else
            {
                d2 = 下d2;
            }

            this.BoundingBoxA = new BoundingBox(new Vector3(-w2, 0, -d2), new Vector3(w2, h, d2));

        */










        // --------------------------------------
        // 外壁のみ表示
        public void Draw簡略1(GraphicsDeviceManager graphics,
                            BasicEffect BasicEffect,     // ライティング+テクスチャ 
                            Matrix view,
                            Matrix proj)
        {
            // 表示切替
            // カメラとの距離が遠い場合、外壁のみ表示
            // 近い場合、外壁を半透明、柱と床を表示

            // 半透明は最後に描画
            this.外壁.pos = this.pos;
            this.外壁.単位行列 = this.単位行列;

            this.外壁.Drawシンプル(graphics, BasicEffect, view, proj);
        }


        // --------------------------------------
        // 外壁 + 等高線 表示
        public void Draw簡略2(  GraphicsDeviceManager graphics,
                                BasicEffect effect,     // ライティング+テクスチャ 
                                Effect effect_simple,  // 等高線表示用 ライティング無し
                                Matrix view,
                                Matrix proj)
        {
            // 表示切替
            // カメラとの距離が遠い場合、外壁のみ表示
            // 近い場合、外壁を半透明、柱と床を表示

            // 半透明は最後に描画
            this.外壁.pos = this.pos;

            this.外壁.単位行列 = this.単位行列;

            this.外壁.Draw(graphics, effect, effect_simple, view, proj);
        }



        // --------------------------------------
        // 外壁輪郭のみ線で表示
        public void Drawライン(GraphicsDeviceManager graphics,
                            Effect effect,  // 線球用 陰線の色を落とす
                            Matrix view,
                            Matrix proj)
        {
            this.外壁.pos = this.pos;
            this.外壁.単位行列 = this.単位行列;

            this.外壁.Drawライン(graphics, effect, view, proj);
        }





        // BoundingBox同士(ビルの外壁同士)の衝突判定を、片方をBoundingFrustumにすることで実現
        // 衝突判定が完全ではない？
        virtual public bool すい台Box判定(ビル box)
        {
            float n = 10000f;

            Matrix view = Matrix.CreateLookAt(
                            box.pos + (box.単位行列.Up * n),    // 視点位置 
                            box.pos,                // 参照点
                            box.単位行列.Forward);  // UPベクトル

            Matrix proj = Matrix.CreatePerspective(
                            box.表示BoundingBox.上w, // w
                            box.表示BoundingBox.上d, // h
                            //n - box.表示BoundingBox.h,  // near
                            n - 16,  // near
                            n );     // far

            Matrix ViewProj = Matrix.Multiply(view, proj);

            // BoundingBox同士の衝突判定を、片方をBoundingFrustumにすることで実現

            Matrix wkM = Matrix.Multiply(this.単位行列, Matrix.CreateTranslation(this.pos));

            BoundingFrustum bf = new BoundingFrustum(Matrix.Multiply(wkM, ViewProj));

            return bf.Intersects(this.外壁.BoundingBoxA);

            // BoundingBoxは動かない。回転しない。BoundingFrustumを動かす。回転して衝突判定
        }




    
    }
}
