﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace ccc4
{

    /// <summary>
    /// ワイヤーフレームで表示され、陰面処理により色分けされた球の作成
    /// 半径、円周の分割数可変
    /// </summary>
    class 線球 : 基本Model
    {
        private float PI = (float)Math.PI;

        private Vector3[,] m_水平線;
        private Vector3[,] m_垂直線;

        private float m_半径;
        private int m_円周分割数;

        private int m_水平数;		// 水平の円の数 円周分割数(引数)/2-1
        private int m_垂直数;		// 垂直の円の数 円周分割数(引数)/2

        private int m_SphereSize = 131072;	// 球描画用の座標数

        private VertexBuffer m_SphereBuffer = null;	// 球

        VertexPositionColor[] verts = null;

        private bool m_b初期回転 = false;	// 初期位置の回転 true=90°回転する。 Unit表示用

        private Color m_col     = new Color( new Vector4(0.0f, 0.0f, 0.0f, 1.0f ) ); // 球の色
        private Color m_col赤道 = new Color( new Vector4(0.0f, 0.0f, 0.0f, 1.0f ) ); // 赤道の色



        // --------------------------------------
        // 以下 xna
        VertexDeclaration       decl = null;



        // --------------------------------------
        /// <summary>
        /// 設定:bool b初期回転 初期位置の回転 true=90°回転 Unit表示用
        /// </summary>
        public 線球(float 半径, int 円周分割数, bool b初期回転, Color col, Color col赤道)
        {
            m_半径 = 半径;

            m_円周分割数 = 円周分割数;

            // m_円周分割数 = 0の場合赤道のみ
            if (0 != m_円周分割数)
                if (2 != m_円周分割数)
                    if (4 != m_円周分割数)
                        if (8 != m_円周分割数)
                            if (16 != m_円周分割数)
                                if (32 != m_円周分割数)
                                    if (64 != m_円周分割数)
                                        if (128 != m_円周分割数)
                                            if (256 != m_円周分割数)
                                            {
                                                m_円周分割数 = 16;	// 0,2,4,8,16,32,64,128,256のどれでもない場合		
                                            }


            m_水平数 = m_円周分割数 / 2 - 1;	// 水平の円の数
            m_垂直数 = m_円周分割数 / 2;		// 垂直の円の数

            // m_円周分割数 = 0の場合赤道のみ
            if (0 == m_円周分割数)
            {
                m_水平数 = 1;   // 水平の円の数
                m_垂直数 = 0;	// 垂直の円の数
            }


            m_水平線 = new Vector3[m_水平数, 129];	// 緯線(横線 水平)
            m_垂直線 = new Vector3[m_垂直数, 129];	// 経線(縦線 垂直)

            m_b初期回転 = b初期回転;

            m_col     = col;        // 色
            m_col赤道 = col赤道;	// 赤道色
        }




        /// <summary>
        /// 作成
        /// </summary>
        public void Initialize( GraphicsDeviceManager graphics )
        {
            decl = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            m_SphereBuffer = new VertexBuffer(
                graphics.GraphicsDevice, 
                typeof(VertexPositionColor), 
                m_SphereSize,
                BufferUsage.WriteOnly );

            verts  = new VertexPositionColor[m_SphereSize];  // m_SphereBuffer


            // 円の頂点初期化　--------------------------------------------------
            float x1 = 0, y1 = m_半径, z1 = 0;
            float x2, y2;
            float x3, y3, z3;

            float sin, cos;

            // ----------------------------------------------------
            // 水平の円
            for (int i = 0; i < m_水平数; i++)
            {
                sin = (float)Math.Sin((PI / (m_水平数 + 1)) * (i + 1));
                cos = (float)Math.Cos((PI / (m_水平数 + 1)) * (i + 1));

                x2 = x1 * cos - y1 * sin;
                y2 = x1 * sin + y1 * cos;

                for (int j = 0; j <= 128; j++)
                {
                    sin = (float)Math.Sin((PI / 64) * j);
                    cos = (float)Math.Cos((PI / 64) * j);

                    x3 = x2 * cos;
                    y3 = x2 * sin;

                    if (m_b初期回転)
                    {
                        // Unitに合わせて90°回転
                        m_水平線[i, j].X =  x3;
                        m_水平線[i, j].Y = -y3;
                        m_水平線[i, j].Z =  y2;
                    }
                    else
                    {
                        m_水平線[i, j].X = x3;
                        m_水平線[i, j].Y = y2;
                        m_水平線[i, j].Z = y3;
                    }
                }
            }

            // ----------------------------------------------------
            // 垂直の円 (上下端を空ける)
            for (int i = 0; i < m_垂直数; i++)
            {
                for (int j = 4; j <= 60; j++)
                {
                    // 1.元になる円を作成
                    sin = (float)Math.Sin((PI / 64) * j);
                    cos = (float)Math.Cos((PI / 64) * j);

                    // z軸を中心に回転
                    x2 = x1 * cos - y1 * sin;
                    y2 = x1 * sin + y1 * cos;

                    // 2.元になる円を回転
                    sin = (float)Math.Sin((PI / m_垂直数) * i);
                    cos = (float)Math.Cos((PI / m_垂直数) * i);

                    // y軸を中心に回転
                    x3 = x2 * cos + z1 * sin;
                    z3 = z1 * cos - x2 * sin;

                    if (m_b初期回転)
                    {
                        // ------------- ===
                        // y軸を中心に45°回転 (unit球用) + -> x になるように
                        sin = (float)Math.Sin((PI / 4));
                        cos = (float)Math.Cos((PI / 4));

                        float x4 = x3 * cos + z3 * sin;
                        float z4 = z3 * cos - x3 * sin;

                        x3 = x4;
                        z3 = z4;
                        // -------------
                        
                        // Unitに合わせて90°回転
                        m_垂直線[i, j - 4].X =  x3;	// -4:上下端を空ける為
                        m_垂直線[i, j - 4].Y = -z3;
                        m_垂直線[i, j - 4].Z =  y2;
                    }
                    else
                    {
                        m_垂直線[i, j - 4].X = x3;		// -4:上下端を空ける為
                        m_垂直線[i, j - 4].Y = y2;
                        m_垂直線[i, j - 4].Z = z3;
                    }
                }


                for (int j = 68; j <= 124; j++)
                {
                    // 1.元になる円を作成
                    sin = (float)Math.Sin((PI / 64) * j);
                    cos = (float)Math.Cos((PI / 64) * j);

                    // z軸を中心に回転
                    x2 = x1 * cos - y1 * sin;
                    y2 = x1 * sin + y1 * cos;

                    // 2.元になる円を回転
                    sin = (float)Math.Sin((PI / m_垂直数) * i);
                    cos = (float)Math.Cos((PI / m_垂直数) * i);

                    // y軸を中心に回転
                    x3 = x2 * cos + z1 * sin;
                    z3 = z1 * cos - x2 * sin;


                    if (m_b初期回転)
                    {
                        // ------------- ===
                        // y軸を中心に45°回転 (unit球用) + -> x になるように
                        sin = (float)Math.Sin((PI / 4));
                        cos = (float)Math.Cos((PI / 4));

                        float x4 = x3 * cos + z3 * sin;
                        float z4 = z3 * cos - x3 * sin;

                        x3 = x4;
                        z3 = z4;
                        // -------------

                        // Unitに合わせて90°回転
                        m_垂直線[i, j - 11].X =  x3;		// -11:上下端を空ける為
                        m_垂直線[i, j - 11].Y = -z3;
                        m_垂直線[i, j - 11].Z =  y2;
                    }
                    else
                    {
                        m_垂直線[i, j - 11].X = x3;		// -11:上下端を空ける為
                        m_垂直線[i, j - 11].Y = y2;
                        m_垂直線[i, j - 11].Z = z3;
                    }
                }
            }
            // ----------------------------------------------------
            /*
            Matrix M11 - M44
            11 12 13 14
            21 22 23 24
            31 32 33 34
            41 42 43 44
            */

            int Count = 0;
            int 赤道 = m_水平数 / 2;


            /////////////////////////////////// 

            // --------------------------------------------------------------------------
            // 緯線(横線 水平)
            for (int i = 0; i < m_水平数; i++)
            {
                for (int j = 0; j <= 128; j++)
                {
                    verts[Count].Position = m_水平線[i, j];

                    // 赤道(中心の円)を色を変えて表示
                    if (赤道 == i)
                    {
                        verts[Count].Color = m_col赤道;
                    }
                    else
                    {
                        verts[Count].Color = m_col;
                    }

                    Count++;
                }
            }


            // --------------------------------------------------------------------------
            // 経線(縦線 垂直) longitude	ランジテュード 
            // (上下端を空ける)
            // 16本の経線で球を32分割

            for (int i = 0; i < m_垂直数; i++)
            {
                for (int j = 0; j <= 113; j++)
                {
                    verts[Count].Position = m_垂直線[i, j];


                    // 色を少し変える
                    verts[Count].Color = new Color( new Vector4( 
                                                        (m_col.R - 0.10f), 
                                                        m_col.G,
                                                        m_col.B, 
                                                        m_col.A ) );
                    // --------------------------------------------------
                    Count++;
                }
            }

            ///////////////////////////////////

            m_SphereBuffer.SetData<VertexPositionColor>(verts);

        }



        /// <summary>
        /// 描画
        /// </summary>
        public void Draw(GraphicsDeviceManager graphics,
                            Effect effect,
                            Matrix view,
                            Matrix proj)
        {
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);  // 回転

            // ワールド * ビュー * 射影行列
            effect.Parameters["WorldViewProj"].SetValue(m * view * proj);

            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            // --------------
            graphics.GraphicsDevice.Vertices[0].SetSource(m_SphereBuffer, 0, VertexPositionColor.SizeInBytes);


            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                // 横の円を描画
                for (int i = 0; i < m_水平数; i++)
                {
                    // プリミティブのシーケンスをレンダリング
                    graphics.GraphicsDevice.DrawPrimitives(
                        PrimitiveType.LineStrip,			// レンダリングするプリミティブの種類
                        i * 129,							// ロードする最初の頂点のインデックス
                        128);								// レンダリングするプリミティブ数(LineStripの場合：線分の本数)
                }

                // 縦の円を描画 (上端、下端を空ける:円が２つ分割)
                for (int i = 0; i < m_垂直数 * 2; i++)
                {
                    // プリミティブのシーケンスをレンダリング
                    graphics.GraphicsDevice.DrawPrimitives(
                        PrimitiveType.LineStrip,			// レンダリングするプリミティブの種類
                        (i * 57) + ((m_垂直数 - 1) * 129),	// ロードする最初の頂点のインデックス
                        56);								// レンダリングするプリミティブ数
                }

                pass.End();
            }
            effect.End();
        }












        /// <summary>
        /// 描画 HLSL用設定はメソッド外で行う
        /// </summary>
        public void Draw2(  GraphicsDeviceManager graphics,
                            Effect effect )
        {
            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            // --------------
            graphics.GraphicsDevice.Vertices[0].SetSource(m_SphereBuffer, 0, VertexPositionColor.SizeInBytes);


            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                // 横の円を描画
                for (int i = 0; i < m_水平数; i++)
                {
                    // プリミティブのシーケンスをレンダリング
                    graphics.GraphicsDevice.DrawPrimitives(
                        PrimitiveType.LineStrip,			// レンダリングするプリミティブの種類
                        i * 129,							// ロードする最初の頂点のインデックス
                        128);								// レンダリングするプリミティブ数(LineStripの場合：線分の本数)
                }

                // 縦の円を描画 (上端、下端を空ける:円が２つ分割)
                for (int i = 0; i < m_垂直数 * 2; i++)
                {
                    // プリミティブのシーケンスをレンダリング
                    graphics.GraphicsDevice.DrawPrimitives(
                        PrimitiveType.LineStrip,			// レンダリングするプリミティブの種類
                        (i * 57) + ((m_垂直数 - 1) * 129),	// ロードする最初の頂点のインデックス
                        56);								// レンダリングするプリミティブ数
                }

                pass.End();
            }
            effect.End();
        }














    } // class 線球xna end

























}
