﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;


namespace ccc4
{

    // ========================================================================
    // unit球 unit表示時に unitAと一緒に表示する線の球
    class unit球 : 基本Model
    {
        VertexDeclaration       decl = null;

        // 線によるx,y,z座標軸+Unit表示
        VertexPositionColor[]   vpcUnit = null;
        VertexBuffer            vbUnit; // Vertex buffer to define a triangle.

        // --------------------------------------
        // 線による射程表示
        // 敵unitとの距離が射程内だったら円を表示
        // 1:射程近
        VertexPositionColor[] vpc射程1 = null;
        VertexBuffer            vb射程1;

        // 2:射程内
        VertexPositionColor[] vpc射程2 = null;
        VertexBuffer            vb射程2;


        線球 unit線球;

        // --------------------------------------
        public unit球()
        {
        }

        // --------------------------------------
        // size : 球の半径
        public void Initialize(GraphicsDeviceManager graphics, float 半径 )
        {
            float size = 半径;

            unit線球 = new 線球(
                                size,   // 半径
                                4,      // 円周分割数 
                                true,
                                new Color(new Vector4(1, 1, 1, 1f)),    // 本体カラー
                                new Color(new Vector4(1, 0, 0, 1f)));   // 赤道カラー

            this.unit線球.Initialize(graphics);

            decl = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            // ---------------------------
            // 線によるx,y,z座標軸+Unit表示 
            size = 半径 / 2f;   // unit球より小さく表示

            vbUnit = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 12, BufferUsage.WriteOnly );
            vpcUnit = new VertexPositionColor[12];
            {
                int i = 0;

                // x軸
                vpcUnit[i].Position = new Vector3(-size, 0,  0);   vpcUnit[i].Color = Color.Red;   i++;
                vpcUnit[i].Position = new Vector3( size, 0,  0);   vpcUnit[i].Color = Color.Red;   i++;

                // y軸 (-側の座標軸を取る)
                vpcUnit[i].Position = new Vector3( 0,    0, 0);   vpcUnit[i].Color = Color.Green;   i++;
                vpcUnit[i].Position = new Vector3( 0, size, 0);   vpcUnit[i].Color = Color.Green;   i++;

                // z軸
                vpcUnit[i].Position = new Vector3( 0, 0, -size);   vpcUnit[i].Color = Color.Blue;   i++;
                vpcUnit[i].Position = new Vector3( 0, 0,  size);   vpcUnit[i].Color = Color.Blue;   i++;

                // x軸 - z軸 アウトライン
                vpcUnit[i].Position = new Vector3(size, 0, 0);  vpcUnit[i].Color = Color.DarkSlateBlue; i++;
                vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.DarkSlateBlue; i++;

                // y軸 - z軸 アウトライン
                vpcUnit[i].Position = new Vector3(0, size, 0); vpcUnit[i].Color = Color.DarkSlateBlue; i++;
                vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.DarkSlateBlue; i++;

                // z軸 - x軸 アウトライン
                vpcUnit[i].Position = new Vector3(-size, 0, 0); vpcUnit[i].Color = Color.DarkSlateBlue; i++;
                vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.DarkSlateBlue; i++;

                vbUnit.SetData<VertexPositionColor>(vpcUnit);
            }



            // ---------------------------
            size = 半径;

            // 線による射程表示
            // 1:射程近
            vb射程1 = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 33, BufferUsage.WriteOnly );
            
            vpc射程1 = new VertexPositionColor[33];
            {

                float 射程表示半径 = 1.5f;

                for (int i = 0; i < 33; i++)
                {
                    float sin = (float)Math.Sin(( Math.PI / 16) * i);
                    float cos = (float)Math.Cos(( Math.PI / 16) * i);

                    float x = 射程表示半径 * cos;
                    float y = 射程表示半径 * sin;

                    vpc射程1[i].Position = new Vector3( x, y, size);
                    vpc射程1[i].Color = Color.Yellow;
                }
                
                vb射程1.SetData<VertexPositionColor>(vpc射程1);
            }
            // ---------------------------
            // 線による射程表示
            // 2:射程内
            vb射程2 = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 33, BufferUsage.WriteOnly);
            vpc射程2 = new VertexPositionColor[33];
            {

                float 射程表示半径 = 1f;

                for (int i = 0; i < 33; i++)
                {
                    float sin = (float)Math.Sin((Math.PI / 16) * i);
                    float cos = (float)Math.Cos((Math.PI / 16) * i);

                    float x = 射程表示半径 * cos;
                    float y = 射程表示半径 * sin;

                    vpc射程2[i].Position = new Vector3(x, y, size);
                    vpc射程2[i].Color = Color.Blue;
                }
                vb射程2.SetData<VertexPositionColor>(vpc射程2);
            }

            



        }



        // --------------------------------------
        public void Draw(   GraphicsDeviceManager graphics,
                            Effect effect,
                            Effect effect2, // 隠れた線、色減衰
                            Matrix view,
                            Matrix proj
                        )
        {
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);  // 回転

            // ワールド * ビュー * 射影行列
            Matrix mViewProj = m * view * proj;
            effect2.Parameters["WorldViewProj"].SetValue(mViewProj);
            effect.Parameters["WorldViewProj"].SetValue(mViewProj);

            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            this.unit線球.pos       = this.pos;
            this.unit線球.単位行列  = this.単位行列;
            this.unit線球.Draw( graphics, effect2, view, proj );

            // --------------
            // 線によるx,y,z座標軸+Unit表示 : 色減衰有り(effect2)
            graphics.GraphicsDevice.Vertices[0].SetSource(vbUnit, 0, VertexPositionColor.SizeInBytes);

            effect2.Begin();
            foreach (EffectPass pass in effect2.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, 6);

                pass.End();
            }
            effect2.End();
        }







        // --------------------------------------
        // 方位を表示するために表示位置を指定して、座標軸を表示する。
        public void Draw方位表示(    GraphicsDeviceManager graphics,
                            Effect effect2, // 隠れた線、色減衰
                            Matrix view,
                            Matrix proj
                        )
        {
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);  // 回転

            // ワールド * ビュー * 射影行列
            Matrix mViewProj = m * view * proj;

            // 表示位置指定
            mViewProj.M41 = 8f; // x 右:大きくすると右へ
            mViewProj.M42 = 8f; // y 上:大きくすると上へ
            mViewProj.M43 = 1f; // z 奥行き:小さくすると手前に表示

            effect2.Parameters["WorldViewProj"].SetValue(mViewProj);

            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            // --------------
            // 線によるx,y,z座標軸+Unit表示 : 色減衰有り(effect2)
            graphics.GraphicsDevice.Vertices[0].SetSource(vbUnit, 0, VertexPositionColor.SizeInBytes);

            effect2.Begin();
            foreach (EffectPass pass in effect2.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, 6);

                pass.End();
            }
            effect2.End();
        }










    } // unit球 end





    // ========================================================================
    // unit影:unitAの影 歩行モード時にunitAの真下に表示
    class unit影 : 基本Model
    {
        VertexDeclaration decl = null;

        // 線によるx, z座標軸+Unit表示
        VertexPositionColor[]   vpcUnit = null;
        VertexBuffer            vbUnit; // Vertex buffer to define a triangle.

        // --------------------------------------
        public unit影()
        {
        }

        
        // --------------------------------------
        // size : 球の半径
        public void Initialize( GraphicsDeviceManager graphics, float 半径 )
        {
            float size = 半径;

            decl = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            // ---------------------------
            // 線によるx,y,z座標軸+Unit表示 
            size = 半径 / 2f;   // unit球より小さく表示

            vbUnit = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 12, BufferUsage.WriteOnly);
            vpcUnit = new VertexPositionColor[8];
            int i = 0;

            // x軸
            vpcUnit[i].Position = new Vector3(-size, 0, 0); vpcUnit[i].Color = Color.Black; i++;
            vpcUnit[i].Position = new Vector3( size, 0, 0); vpcUnit[i].Color = Color.Black; i++;

            // z軸
            vpcUnit[i].Position = new Vector3(0, 0, -size); vpcUnit[i].Color = Color.Black; i++;
            vpcUnit[i].Position = new Vector3(0, 0,  size); vpcUnit[i].Color = Color.Black; i++;

            // x軸 - z軸 アウトライン
            vpcUnit[i].Position = new Vector3(size, 0, 0); vpcUnit[i].Color = Color.Black; i++;
            vpcUnit[i].Position = new Vector3(0, 0, size); vpcUnit[i].Color = Color.Black; i++;

            // z軸 - x軸 アウトライン
            vpcUnit[i].Position = new Vector3(-size, 0, 0); vpcUnit[i].Color = Color.Black; i++;
            vpcUnit[i].Position = new Vector3(0, 0, size);  vpcUnit[i].Color = Color.Black; i++;

            vbUnit.SetData<VertexPositionColor>(vpcUnit);
        }


        // --------------------------------------
        public void Draw(   GraphicsDeviceManager graphics,
                            Effect effect2, // 隠れた線、色減衰
                            Matrix view,
                            Matrix proj
                        )
        {
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);  // 回転

            // ワールド * ビュー * 射影行列
            Matrix mViewProj = m * view * proj;
            effect2.Parameters["WorldViewProj"].SetValue(mViewProj);

            graphics.GraphicsDevice.VertexDeclaration = this.decl;

            // --------------
            // 線によるx,y,z座標軸+Unit表示 : 色減衰有り(effect2)
            graphics.GraphicsDevice.Vertices[0].SetSource(vbUnit, 0, VertexPositionColor.SizeInBytes);

            effect2.Begin();
            foreach (EffectPass pass in effect2.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.LineList, 0, 4);

                pass.End();
            }
            effect2.End();
        }
    
    } // unit影 end






    // ========================================================================
    // 正４面体表示
    class 正4面体 : 基本Model
    {

        VertexDeclaration       decl = null;
        VertexPositionColor[]   vpc  = null;
        VertexBuffer            vb; // Vertex buffer to define a triangle.

        // --------------------------------------
        public 正4面体()
        {
        }

        // --------------------------------------
        // 正4面体 size = 辺の長さ
        public void Initialize(GraphicsDeviceManager graphics, float size)
        {            
            decl = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);
            vb = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionColor), 12, BufferUsage.WriteOnly);
            vpc  = new VertexPositionColor[12];

            float size2 = size / 2f;

            float t = ( (float)Math.Sqrt(2) / 2 ) * size2;  // 正四面体の高さ

            Vector3 p0 = new Vector3(-size2,  t,      0 );
            Vector3 p1 = new Vector3( size2,  t,      0 );
            Vector3 p2 = new Vector3( 0,     -t, -size2 );
            Vector3 p3 = new Vector3( 0,     -t,  size2 );


            // 右回り(時計回り)
            vpc[ 0].Position = p0;
            vpc[ 1].Position = p1;
            vpc[ 2].Position = p2;

            vpc[ 3].Position = p1;
            vpc[ 4].Position = p0;
            vpc[ 5].Position = p3;

            vpc[ 6].Position = p1;
            vpc[ 7].Position = p3;
            vpc[ 8].Position = p2;

            vpc[ 9].Position = p0;
            vpc[10].Position = p2;
            vpc[11].Position = p3;


/*
            vpc[ 0].Color = new Color(255,   0,   0);
            vpc[ 1].Color = new Color(200,   0,   0);
            vpc[ 2].Color = new Color(150,   0,   0);

            vpc[ 3].Color = new Color(  0, 255,   0);
            vpc[ 4].Color = new Color(  0, 200,   0);
            vpc[ 5].Color = new Color(  0, 150,   0);

            vpc[ 6].Color = new Color(  0,   0, 255);
            vpc[ 7].Color = new Color(  0,   0, 200);
            vpc[ 8].Color = new Color(  0,   0, 150);

            vpc[ 9].Color = new Color(255,   0, 255);
            vpc[10].Color = new Color(200,   0, 200);
            vpc[11].Color = new Color(150,   0, 150);
 */

/*
            vpc[ 0].Color = new Color(192, 192, 192);
            vpc[ 1].Color = new Color(137, 137, 137);
            vpc[ 2].Color = new Color(142, 142, 142);

            vpc[ 3].Color = new Color(192, 192, 192);
            vpc[ 4].Color = new Color(137, 137, 137);
            vpc[ 5].Color = new Color(142, 142, 142);

            vpc[ 6].Color = new Color(128, 128, 128);
            vpc[ 7].Color = new Color( 73,  73,  73);
            vpc[ 8].Color = new Color( 23,  23,  23);

            vpc[ 9].Color = new Color(128, 128, 128);
            vpc[10].Color = new Color( 73,  73,  73);
            vpc[11].Color = new Color( 23,  23,  23);
*/

            vpc[ 0].Color = new Color(128, 128, 128);
            vpc[ 1].Color = new Color( 73,  73,  73);
            vpc[ 2].Color = new Color( 23,  23,  23);

            vpc[ 3].Color = new Color(128, 128, 128);
            vpc[ 4].Color = new Color( 73,  73,  73);
            vpc[ 5].Color = new Color( 23,  23,  23);

            vpc[ 6].Color = new Color(128, 128, 128);
            vpc[ 7].Color = new Color( 73,  73,  73);
            vpc[ 8].Color = new Color( 23,  23,  23);

            vpc[ 9].Color = new Color(128, 128, 128);
            vpc[10].Color = new Color( 73,  73,  73);
            vpc[11].Color = new Color( 23,  23,  23);


 
            vb.SetData<VertexPositionColor>(vpc);

        }



        // --------------------------------------
        public void Draw(   GraphicsDeviceManager graphics,
                            Effect effect,
                            Matrix view,
                            Matrix proj )
        {
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);  // 回転
            
            // ワールド * ビュー * 射影行列
            effect.Parameters["WorldViewProj"].SetValue( m * view * proj );

            graphics.GraphicsDevice.VertexDeclaration = this.decl;
            graphics.GraphicsDevice.Vertices[0].SetSource( vb, 0, VertexPositionColor.SizeInBytes);

            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                
                graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 4 );

                pass.End();
            }
            effect.End();
        }







        // --------------------------------------
        // size * size * size のグリッド表示、原点が中心
        // 注意!! 重いメソッド
        public void Drawグリッド(   GraphicsDeviceManager graphics,
                                    Effect  effect,
                                    Matrix  view,
                                    Matrix  proj,
                                    int     size,
                                    float   グリッド間隔,
                                    Vector3 グリッドpos 
                                    )
        {
            Matrix  m;
            Matrix  ViewPproj = view * proj;

            Vector3 wk_pos;

            graphics.GraphicsDevice.VertexDeclaration       = this.decl;

            graphics.GraphicsDevice.Vertices[0].SetSource(vb, 0, VertexPositionColor.SizeInBytes);

            // Boxグリッド表示(8*8*8)
            float グリッド間隔調整 = グリッド間隔 * ((size - 1) * 0.5f);   // グリッド中心を原点に移動


            for (int i = 0; i < size; i++)
                for (int j = 0; j < size; j++)
                    for (int k = 0; k < size; k++)
                    {
                        // ================================================================
                        wk_pos = new Vector3(   (グリッド間隔 * i) - グリッド間隔調整,
                                                (グリッド間隔 * j) - グリッド間隔調整,
                                                (グリッド間隔 * k) - グリッド間隔調整 );

                        wk_pos += グリッドpos;

//                      m = this.単位行列 * Matrix.CreateTranslation(wk_pos);   // 回転
                        m = Matrix.CreateTranslation(wk_pos);   // 回転 (角度固定:単位行列の乗算、省略)

                        // ワールド * ビュー * 射影行列
//                      effect.Parameters["WorldViewProj"].SetValue(m * view * projection);
                        effect.Parameters["WorldViewProj"].SetValue(m * ViewPproj );

                        effect.Begin();
                        foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                        {
                            pass.Begin();

                            graphics.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 4);

                            pass.End();
                        }
                        effect.End();
                    }
        }
        // --------------------------------------

    
    }





    
    // ========================================================================
    // 正６面体表示
    // 半径指定
    class Box : 基本Model
    {
        VertexDeclaration       cubeVertexDeclaration;
        VertexPositionColor[]   cubeVertices;
        short[]                 cubeIndices;
        VertexBuffer            vertexBuffer;
        IndexBuffer             indexBuffer;


        // --------------------------------------
        public Box()
        {
        }


        // --------------------------------------
        // 正６面体 size = 半径
        public void Initialize(GraphicsDeviceManager graphics, float size)
        {
            cubeVertexDeclaration = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            cubeVertices = new VertexPositionColor[24];

            Vector3 topLeftFront        = new Vector3(-size,  size,  size);
            Vector3 bottomLeftFront     = new Vector3(-size, -size,  size);
            Vector3 topRightFront       = new Vector3( size,  size,  size);
            Vector3 bottomRightFront    = new Vector3( size, -size,  size);
            Vector3 topLeftBack         = new Vector3(-size,  size, -size);
            Vector3 topRightBack        = new Vector3( size,  size, -size);
            Vector3 bottomLeftBack      = new Vector3(-size, -size, -size);
            Vector3 bottomRightBack     = new Vector3( size, -size, -size);

            cubeVertices[ 0] = new VertexPositionColor(topLeftFront, Color.Red);
            cubeVertices[ 1] = new VertexPositionColor(bottomLeftFront, Color.Red);
            cubeVertices[ 2] = new VertexPositionColor(topRightFront, Color.Red);
            cubeVertices[ 3] = new VertexPositionColor(bottomRightFront, Color.Red);

            cubeVertices[ 4] = new VertexPositionColor(topLeftBack, Color.Orange);
            cubeVertices[ 5] = new VertexPositionColor(topRightBack, Color.Orange);
            cubeVertices[ 6] = new VertexPositionColor(bottomLeftBack, Color.Orange);
            cubeVertices[ 7] = new VertexPositionColor(bottomRightBack, Color.Orange);

            cubeVertices[ 8] = new VertexPositionColor(topLeftFront, Color.Yellow);
            cubeVertices[ 9] = new VertexPositionColor(topRightBack, Color.Yellow);
            cubeVertices[10] = new VertexPositionColor(topLeftBack, Color.Yellow);
            cubeVertices[11] = new VertexPositionColor(topRightFront, Color.Yellow);

            cubeVertices[12] = new VertexPositionColor(bottomLeftFront, Color.Purple);
            cubeVertices[13] = new VertexPositionColor(bottomLeftBack, Color.Purple);
            cubeVertices[14] = new VertexPositionColor(bottomRightBack, Color.Purple);
            cubeVertices[15] = new VertexPositionColor(bottomRightFront, Color.Purple);

            cubeVertices[16] = new VertexPositionColor(topLeftFront, Color.Blue);
            cubeVertices[17] = new VertexPositionColor(bottomLeftBack, Color.Blue);
            cubeVertices[18] = new VertexPositionColor(bottomLeftFront, Color.Blue);
            cubeVertices[19] = new VertexPositionColor(topLeftBack, Color.Blue);

            cubeVertices[20] = new VertexPositionColor(topRightFront, Color.Green);
            cubeVertices[21] = new VertexPositionColor(bottomRightFront, Color.Green);
            cubeVertices[22] = new VertexPositionColor(bottomRightBack, Color.Green);
            cubeVertices[23] = new VertexPositionColor(topRightBack, Color.Green);

            cubeIndices = new short[] {  0,  1,  2,  // red front face
                                         1,  3,  2,
                                         4,  5,  6,  // orange back face
                                         6,  5,  7,
                                         8,  9, 10,  // yellow top face
                                         8, 11,  9,
                                        12, 13, 14,  // purple bottom face
                                        12, 14, 15,
                                        16, 17, 18,  // blue left face
                                        19, 17, 16,
                                        20, 21, 22, // green right face
                                        23, 20, 22  };

            vertexBuffer = new VertexBuffer(
                graphics.GraphicsDevice,
                cubeVertices.Length * VertexPositionColor.SizeInBytes,
                BufferUsage.None );

            vertexBuffer.SetData<VertexPositionColor>(cubeVertices);

            indexBuffer = new IndexBuffer(
                graphics.GraphicsDevice,
                sizeof(short) * cubeIndices.Length,
                BufferUsage.None,
                IndexElementSize.SixteenBits );

            indexBuffer.SetData<short>(cubeIndices);
        }



        // --------------------------------------
        public void Draw(   GraphicsDeviceManager graphics,
                            Effect effect,
                            Matrix view,
                            Matrix proj )
        {
            Matrix m = this.単位行列 * Matrix.CreateTranslation(this.pos);              // 回転
            
            // ワールド * ビュー * 射影行列
            effect.Parameters["WorldViewProj"].SetValue( m * view * proj );

            graphics.GraphicsDevice.VertexDeclaration = cubeVertexDeclaration;
            graphics.GraphicsDevice.Indices = indexBuffer;
            graphics.GraphicsDevice.Vertices[0].SetSource(vertexBuffer, 0, VertexPositionColor.SizeInBytes);

            // effect is a compiled effect created and compiled elsewhere
            // in the application

            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawIndexedPrimitives(
                    PrimitiveType.TriangleList,
                    0,
                    0,
                    cubeVertices.Length,
                    0,
                    12 );

                pass.End();
            }
            effect.End();
        
        }



        // --------------------------------------
        // size * size * size のグリッド表示、原点が中心
        public void Drawグリッド(
                            GraphicsDeviceManager graphics, Effect effect,
                            Matrix world,
                            Matrix view,
                            Matrix proj,
                            int size, float グリッド間隔 )
        {
            Matrix  m;
            Vector3 pos;

            graphics.GraphicsDevice.VertexDeclaration       = cubeVertexDeclaration;
            graphics.GraphicsDevice.Indices                 = indexBuffer;
            graphics.GraphicsDevice.Vertices[0].SetSource( vertexBuffer, 0, VertexPositionColor.SizeInBytes );
            
            // Boxグリッド表示(8*8*8)
            float グリッド間隔調整 = グリッド間隔 * ( (size-1) * 0.5f );   // グリッド中心を原点に移動

            for (int i = 0; i < size; i++)
                for (int j = 0; j < size; j++)
                    for (int k = 0; k < size; k++)
                    {
                        // ================================================================
                        pos = new Vector3(  (グリッド間隔 * i) - グリッド間隔調整,
                                            (グリッド間隔 * j) - グリッド間隔調整,
                                            (グリッド間隔 * k) - グリッド間隔調整 );

                        m = this.単位行列 * Matrix.CreateTranslation(pos) * world;              // 回転

                        // ワールド * ビュー * 射影行列
                        effect.Parameters["WorldViewProj"].SetValue(m * view * proj);

                        effect.Begin();
                        foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                        {
                            pass.Begin();

                            graphics.GraphicsDevice.DrawIndexedPrimitives(
                                PrimitiveType.TriangleList,
                                0,
                                0,
                                cubeVertices.Length,
                                0,
                                12);

                            pass.End();
                        }
                        effect.End();

                    }
        }
        // --------------------------------------
    }





    // ========================================================================
    // モデル表示(.x / .fbx ファイル)
    class xModel : 基本Model
    {
        private Model model = new Model();      // モデル

        // --------------------------------------
        public xModel()
		{
        }

        // --------------------------------------
        public void Initialize(ContentManager content, string assetName)
        {
            model = content.Load<Model>(assetName);
        }




        // --------------------------------------
        public void Draw(   GraphicsDeviceManager graphics,
                            Matrix view,
                            Matrix proj )
        {
            // 3Dモデルの描画
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;  // 両面描画(モデルは両面描画が必要?)

            foreach (ModelMesh mesh in model.Meshes)
            {
                //エフェクトの指定
                foreach (BasicEffect ModelEffect in mesh.Effects)
                {
                    // ライティング
                  ModelEffect.EnableDefaultLighting();

                  // アンビエント光 (環境光)
                  // アンビエント光は、光が拡散し、すべてのオブジェクトを一様に照らすライト。
                  ModelEffect.AmbientLightColor = new Vector3(0.5f, 0.5f, 0.5f);

ModelEffect.DirectionalLight0.Direction = new Vector3(0,1,0);

/*
スペキュラ光
specular light
鏡面反射光
物体に光があたったときに、最も強くその光を反射している部分の光の総称。
*/

/*
デフューズ光
diffuse light
拡散反射光
物体にあたった光はその内部に取り込まれ、反射を繰り返して拡散光となってふたたび外に出る。この拡散光の総称。
*/

// ---------------------
/*
ModelEffect.Alpha = 1.0f;

ModelEffect.DiffuseColor = new Vector3(0.8f, 0.0f, 0.0f);

ModelEffect.SpecularColor = new Vector3(0f, 0f, 0f);
ModelEffect.SpecularPower = 5.0f;

ModelEffect.AmbientLightColor = new Vector3(0.05f, 0.09f, 0.18f);
*/

                    // ビュー変換行列
                    ModelEffect.View = view;

                    //射影変換行列
                    ModelEffect.Projection = proj;

                    //ワールド変換行列
                    ModelEffect.World = this.単位行列 *
                                        mesh.ParentBone.Transform *
                                        Matrix.CreateTranslation( this.pos ); //モデル位置
                }
                //メッシュの描画
                mesh.Draw();

            }
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;  // CullMode元に
        }
        // --------------------------------------







        
        // --------------------------------------
        // ユニット歩行時 左右回転、加速減速で少しロール
        public void Draw歩行unit(   GraphicsDeviceManager graphics,
                                    Matrix 操作,
                                    Matrix view,
                                    Matrix proj )
        {
            // 3Dモデルの描画
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;  // 両面描画(モデルは両面描画が必要?)

            foreach (ModelMesh mesh in model.Meshes)
            {
                //エフェクトの指定
                foreach (BasicEffect ModelEffect in mesh.Effects)
                {
                    // ライティング
                    ModelEffect.EnableDefaultLighting();

                    // アンビエント光 (環境光)
                    // アンビエント光は、光が拡散し、すべてのオブジェクトを一様に照らすライト。
                    ModelEffect.AmbientLightColor = new Vector3(0.5f, 0.5f, 0.5f);

                    // ビュー変換行列
                    ModelEffect.View = view;

                    //射影変換行列
                    ModelEffect.Projection = proj;

                    //ワールド変換行列
//                  ModelEffect.World = this.単位行列 * 
                    ModelEffect.World = this.単位行列 * 操作 *
                                        mesh.ParentBone.Transform *
                                        Matrix.CreateTranslation(this.pos); //モデル位置
                }
                //メッシュの描画
                mesh.Draw();
            }
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;  // CullMode元に
        }



        












        // --------------------------------------
        // xModel
        // マウスオーバー,ピックの判定
        // 戻り値 : 交差無し=null / 交差するまでの距離=交差有り
        // xModelクラスではBoundingBoxを使用しない。meshのBoundingSphereを使用
        override public float? ピック判定(GraphicsDeviceManager graphics, // xModel用
                                            Matrix view, Matrix proj,
                                            int mouseX, int mouseY )
        {
            // マウスカーソル位置からのレイを作成
            Vector3 nearsource = new Vector3((float)mouseX, (float)mouseY, 0f);
            Vector3 farsource  = new Vector3((float)mouseX, (float)mouseY, 1f);

            Matrix world = this.単位行列 * Matrix.CreateTranslation( this.pos);

            Vector3 nearPoint = graphics.GraphicsDevice.Viewport.Unproject(nearsource, proj, view, world);
            Vector3 farPoint  = graphics.GraphicsDevice.Viewport.Unproject(farsource,  proj, view, world);

            // Create a ray from the near clip plane to the far clip plane.
            Vector3 direction = farPoint - nearPoint;
            direction.Normalize();

            Ray pickRay = new Ray(nearPoint, direction);

            foreach (ModelMesh mesh in model.Meshes)
            {
                // 透視変換をしてから衝突判定
                Nullable<float> result = pickRay.Intersects(mesh.BoundingSphere);
                if (result != null)
                {
                    return result;  // 交差有りの場合:交差するまでの距離
                }
            }
            return null;    // 交差無しの場合
        }
        // --------------------------------------


        // xModelクラスではレイ判定は行わない。
        override public float? レイ判定(Ray ray)
        {
            return null;    // 交差無しの場合
        }






        // --------------------------------------
        // アンビエント光 指定
        public void Drawアンビエント(   GraphicsDeviceManager graphics,
                            Matrix  view,
                            Matrix  proj,
                            Vector3 アンビエント光 )
        {
            // 3Dモデルの描画
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;  // 両面描画(モデルは両面描画が必要?)

            foreach (ModelMesh mesh in model.Meshes)
            {
                //エフェクトの指定
                foreach (BasicEffect ModelEffect in mesh.Effects)
                {
                    // ライティング
                    ModelEffect.EnableDefaultLighting();

                    // アンビエント光 (環境光)
                    // アンビエント光は、光が拡散し、すべてのオブジェクトを一様に照らすライト。
                    ModelEffect.AmbientLightColor = アンビエント光;

                    // ビュー変換行列
                    ModelEffect.View = view;

                    //射影変換行列
                    ModelEffect.Projection = proj;

                    //ワールド変換行列
                    ModelEffect.World = this.単位行列 *
                                        mesh.ParentBone.Transform *
                                        Matrix.CreateTranslation(this.pos); //モデル位置
                }
                //メッシュの描画
                mesh.Draw();
            }
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;  // CullMode元に

        }
        // --------------------------------------





        // --------------------------------------
        // 半透明モデル表示
        public void DrawAlpha(  GraphicsDeviceManager graphics,
                                Matrix view,
                                Matrix proj )
        {
            // 3Dモデルの描画

            // -----------------------------------------------
            // 半透明表示 設定

            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullCounterClockwiseFace; // 半透明の時はCullMode変更

            graphics.GraphicsDevice.RenderState.SourceBlend      = Blend.SourceAlpha;
            graphics.GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;

            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = true;    // 半透明表示開始
            // -----------------------------------------------

            foreach (ModelMesh mesh in model.Meshes)
            {
                //エフェクトの指定
                foreach (BasicEffect ModelEffect in mesh.Effects)
                {
                    // ライティング
                    ModelEffect.EnableDefaultLighting();

                    // アンビエント光 (環境光)
                    // アンビエント光は、光が拡散し、すべてのオブジェクトを一様に照らすライト。
                    ModelEffect.AmbientLightColor = new Vector3(0.5f, 0.5f, 0.5f);

                        // ビュー変換行列
                    ModelEffect.View = view;

                    //射影変換行列
                    ModelEffect.Projection = proj;

                    //ワールド変換行列
                    ModelEffect.World = this.単位行列 *
                                        mesh.ParentBone.Transform *
                                        Matrix.CreateTranslation(this.pos); //モデル位置
                }
                //メッシュの描画
                mesh.Draw();
            }

            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;  // CullMode元に
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = false;   // 半透明表示終了
        
        }
        // --------------------------------------



    }

    


    // ========================================================================
    public class 基本Model
    {
        private Matrix  m_単位行列  = Matrix.Identity;  // モデル単位行列
        private Vector3 m_pos       = Vector3.Zero;     // モデル位置

        // ビルボード処理用
        // 歩行unitがBoundingBoxを持つ + 歩行unitからレイを飛ばす。
        private BoundingBox m_BoundingBoxA = new BoundingBox();

        // BoundingBox
        [System.Xml.Serialization.XmlIgnoreAttribute]
        public BoundingBox BoundingBoxA
        {
            get { return m_BoundingBoxA; }
            set { m_BoundingBoxA = value; }
        }


        // アクセサ ----------------------------------------------------------------------
        // 単位行列
        public Matrix 単位行列
        {
            get { return m_単位行列; }
            set { m_単位行列 = value; }
        }
        /*
        Matrix M11 - M44
        11 12 13 14 -X
        21 22 23 24 -Y
        31 32 33 34 -Z
        41 42 43 44
        */



        // 位置座標
        public Vector3 pos
        {
            get { return m_pos; }
            set { m_pos = value; }
        }


        public 基本Model()
		{
            m_単位行列 = Matrix.Identity;  // モデル単位行列
            m_pos = Vector3.Zero;     // モデル位置
            m_BoundingBoxA = new BoundingBox();
        }

        // ====================================================================
        // ピッチ(x軸の回転)
        // d:回転角(ラジアン)
        public void RotateX_Pitch(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Right, d);

            単位行列正規化();   // 一定間隔毎にunitの単位行列の正規化
        }


        // ヨー(y軸の回転)
        public void RotateY_Yaw(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Up, d);

            単位行列正規化();   // 一定間隔毎にunitの単位行列の正規化
        }

        
        // ロール(z軸の回転)
        public void RotateZ_Roll(float d)
        {
            if (0f == d)
            {
                return;
            }

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Backward, d);

            単位行列正規化();   // 一定間隔毎にunitの単位行列の正規化
        }





        //	直線移動
        public void 直進(float 移動量)
        {
            if ( 0f == 移動量 )
            {
                return;
            }

            // -------------------------------------------
            m_pos += m_単位行列.Backward * 移動量;
        }

        public void 横移動(float 移動量)
        {
            if (0f == 移動量)
            {
                return;
            }

            // -------------------------------------------
            m_pos += m_単位行列.Left * 移動量;
        }



        // ===============================================================
        // ===============================================================
        // 球の表面をユニットが移動
        // ユニットの位置 : m_単位行列.Up(m.M21, m.M22, m.M23) * 球の半径
        // ---------------------------------------------------------------
        // d:回転角(ラジアン)



        /// <summary>
        /// 球上を直進 QQQ
        /// </summary>
        /// <param name="移動距離"></param>
        /// <param name="球半径"></param>
        public void 直進_Q( float 移動距離 )
        {
            if (0f == 移動距離)
            {
                return;
            }

            float 球半径 = this.pos.Length();

            float d = 移動距離 / 球半径;   // 単位行列回転角

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Right, d); // m_単位行列.Right = 回転の軸を表すベクトル

            単位行列正規化_Q();   // 一定間隔毎に球上のunitの単位行列の正規化

            this.pos = this.m_単位行列.Up * 球半径;
        }


        /// <summary>
        /// 球上を左右 横移動
        /// </summary>
        /// <param name="移動距離"></param>
        /// <param name="球半径"></param>
        public void 横移動_Q(float 移動距離 )
        {
            if (0f == 移動距離)
            {
                return;
            }

            float 球半径 = this.pos.Length();

            float d = 移動距離 / 球半径;   // 単位行列回転角

            m_単位行列 *= Matrix.CreateFromAxisAngle(m_単位行列.Backward, d); // m_単位行列.Backward = 回転の軸を表すベクトル

            単位行列正規化_Q();   // 一定間隔毎に球上のunitの単位行列の正規化

            this.pos = this.m_単位行列.Up * 球半径;
        }


        /// <summary>
        /// 法線設定 
        /// </summary>
        /// <param name="p0"></param>
        /// <param name="p1"></param>
        /// <param name="p2"></param>
        /// <returns></returns>
        public Vector3 NormalFromTriangle_法線設定2(Vector3 p0, Vector3 p1, Vector3 p2)
        {
            // Computes a normal from three triangle vertices
            Vector3 v0 = p1 - p2;
            Vector3 v1 = p0 - p2;

            Vector3 normal = Vector3.Cross(v0, v1);

            normal.Normalize();

            return normal;
        }





        int 単位行列正規化カウンタ_Q = 0;

        /// <summary>
        /// 一定間隔毎に "球上" のunitの単位行列の正規化
        /// </summary>
        private void 単位行列正規化_Q()
        {
            単位行列正規化カウンタ_Q++;

            if (2048 < 単位行列正規化カウンタ_Q)
            {
                float angX, angY, angZ;

                Matrix wkM, wkMA;

                wkM = Matrix.Identity;
                wkM.Up = this.pos;
                wkM.Backward = this.単位行列.Backward; // unitの進行方向(単位行列のBackward)

                // AbsX > AbsZ:z軸角度から求める。
                // AbsX < AbsZ:x軸角度から求める。

                if (Math.Abs(this.pos.X) < Math.Abs(this.pos.Z))
                {
                    // --------------------------------------------------------------------------------
                    angX = (float)Math.Atan2(this.pos.Z, this.pos.Y);
                    wkM *= Matrix.CreateRotationX(-angX);   // x 回転

                    // --------------------------------------------------------------------------------
                    angZ = (float)Math.Atan2(wkM.Up.X, wkM.Up.Y);
                    wkM *= Matrix.CreateRotationZ(angZ);    // z 回転

                    // --------------------------------------------------------------------------------
                    angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                    // 新しい単位行列
                    wkMA  = Matrix.CreateRotationY( angY);
                    wkMA *= Matrix.CreateRotationZ(-angZ);
                    wkMA *= Matrix.CreateRotationX( angX);
                }
                else
                {
                    // --------------------------------------------------------------------------------
                    angZ = (float)Math.Atan2(this.pos.X, this.pos.Y);
                    wkM *= Matrix.CreateRotationZ(angZ);   // z 回転

                    // --------------------------------------------------------------------------------
                    angX = (float)Math.Atan2(wkM.Up.Z, wkM.Up.Y);
                    wkM *= Matrix.CreateRotationX(-angX);    // x 回転

                    // --------------------------------------------------------------------------------
                    angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                    // 新しい単位行列
                    wkMA  = Matrix.CreateRotationY( angY);
                    wkMA *= Matrix.CreateRotationX( angX);
                    wkMA *= Matrix.CreateRotationZ(-angZ);
                }
                // ===============================================================
                // ===============================================================

                this.単位行列 = wkMA;

                単位行列正規化カウンタ_Q = 0;
            }
        }




        int 単位行列正規化カウンタ = 0;

        /// <summary>
        /// 一定間隔毎にunitの単位行列の正規化
        /// </summary>
        private void 単位行列正規化()
        {
            単位行列正規化カウンタ++;

            if (2048 < 単位行列正規化カウンタ)
            {
                float angX, angY, angZ;

                Matrix wkM, wkMA;

                wkM = this.単位行列;

                // AbsX > AbsZ:z軸角度から求める。
                // AbsX < AbsZ:x軸角度から求める。

                if (Math.Abs(this.単位行列.Up.X) < Math.Abs(this.単位行列.Up.Z))
                {
                    // --------------------------------------------------------------------------------
                    angX = (float)Math.Atan2(this.単位行列.Up.Z, this.単位行列.Up.Y);
                    wkM *= Matrix.CreateRotationX(-angX);   // x 回転

                    // --------------------------------------------------------------------------------
                    angZ = (float)Math.Atan2(wkM.Up.X, wkM.Up.Y);
                    wkM *= Matrix.CreateRotationZ(angZ);    // z 回転

                    // --------------------------------------------------------------------------------
                    angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                    // 新しい単位行列
                    wkMA  = Matrix.CreateRotationY( angY);
                    wkMA *= Matrix.CreateRotationZ(-angZ);
                    wkMA *= Matrix.CreateRotationX( angX);
                }
                else
                {
                    // --------------------------------------------------------------------------------
                    angZ = (float)Math.Atan2(this.単位行列.Up.X, this.単位行列.Up.Y);
                    wkM *= Matrix.CreateRotationZ(angZ);   // z 回転

                    // --------------------------------------------------------------------------------
                    angX = (float)Math.Atan2(wkM.Up.Z, wkM.Up.Y);
                    wkM *= Matrix.CreateRotationX(-angX);    // x 回転

                    // --------------------------------------------------------------------------------
                    angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                    // 新しい単位行列
                    wkMA  = Matrix.CreateRotationY( angY);
                    wkMA *= Matrix.CreateRotationX( angX);
                    wkMA *= Matrix.CreateRotationZ(-angZ);
                }
                // ===============================================================
                // ===============================================================

                this.単位行列 = wkMA;

                単位行列正規化カウンタ = 0;
            }
        }




        

        /// <summary>
        /// 球に対して水平になる単位行列を求める。
        /// </summary>
        /// <returns></returns>
        public Matrix 水平姿勢() 
        {
            float angX, angY, angZ;

            Matrix wkM, wkMA;

            wkM = Matrix.Identity;
            wkM.Up = this.pos;
            wkM.Backward = this.単位行列.Backward; // unitの進行方向(単位行列のBackward)

            // AbsX > AbsZ:z軸角度から求める。
            // AbsX < AbsZ:x軸角度から求める。
            if (Math.Abs(this.pos.X) < Math.Abs(this.pos.Z))
            {
                // --------------------------------------------------------------------------------
                angX = (float)Math.Atan2(this.pos.Z, this.pos.Y);
                wkM *= Matrix.CreateRotationX(-angX);   // x 回転

                // --------------------------------------------------------------------------------
                angZ = (float)Math.Atan2(wkM.Up.X, wkM.Up.Y);
                wkM *= Matrix.CreateRotationZ(angZ);    // z 回転

                // --------------------------------------------------------------------------------
                angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                // 新しい単位行列
                wkMA  = Matrix.CreateRotationY( angY);
                wkMA *= Matrix.CreateRotationZ(-angZ);
                wkMA *= Matrix.CreateRotationX( angX);
            }
            else
            {
                // --------------------------------------------------------------------------------
                angZ = (float)Math.Atan2(this.pos.X, this.pos.Y);
                wkM *= Matrix.CreateRotationZ(angZ);   // z 回転

                // --------------------------------------------------------------------------------
                angX = (float)Math.Atan2(wkM.Up.Z, wkM.Up.Y);
                wkM *= Matrix.CreateRotationX(-angX);    // x 回転

                // --------------------------------------------------------------------------------
                angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                // 新しい単位行列
                wkMA  = Matrix.CreateRotationY( angY);
                wkMA *= Matrix.CreateRotationX( angX);
                wkMA *= Matrix.CreateRotationZ(-angZ);
            }

            return wkMA;
        }






        /// <summary>
        /// 球上の一点において、球に対して水平になる単位行列を求める。(ユニットの上下をビルに合わせる)
        /// </summary>
        /// <returns></returns>
        public Matrix 水平姿勢_ビル(Vector3 Pos)
        {
            float angX, angY, angZ;

            Matrix wkM, wkMA;

            wkM = Matrix.Identity;
            wkM.Up = Pos;
            wkM.Backward = this.単位行列.Backward; // unitの進行方向(単位行列のBackward)

            // AbsX > AbsZ:z軸角度から求める。
            // AbsX < AbsZ:x軸角度から求める。
            if (Math.Abs(Pos.X) < Math.Abs(Pos.Z))
            {
                // --------------------------------------------------------------------------------
                angX = (float)Math.Atan2(Pos.Z, Pos.Y);
                wkM *= Matrix.CreateRotationX(-angX);   // x 回転

                // --------------------------------------------------------------------------------
                angZ = (float)Math.Atan2(wkM.Up.X, wkM.Up.Y);
                wkM *= Matrix.CreateRotationZ(angZ);    // z 回転

                // --------------------------------------------------------------------------------
                angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                // 新しい単位行列
                wkMA = Matrix.CreateRotationY(angY);
                wkMA *= Matrix.CreateRotationZ(-angZ);
                wkMA *= Matrix.CreateRotationX(angX);
            }
            else
            {
                // --------------------------------------------------------------------------------
                angZ = (float)Math.Atan2(Pos.X, Pos.Y);
                wkM *= Matrix.CreateRotationZ(angZ);   // z 回転

                // --------------------------------------------------------------------------------
                angX = (float)Math.Atan2(wkM.Up.Z, wkM.Up.Y);
                wkM *= Matrix.CreateRotationX(-angX);    // x 回転

                // --------------------------------------------------------------------------------
                angY = (float)Math.Atan2(wkM.Backward.X, wkM.Backward.Z);

                // 新しい単位行列
                wkMA = Matrix.CreateRotationY(angY);
                wkMA *= Matrix.CreateRotationX(angX);
                wkMA *= Matrix.CreateRotationZ(-angZ);
            }

            return wkMA;
        }







        // --------------------------------------
        /// <summary>
        /// マウスオーバー,ピックの判定
        /// 戻り値 : 交差無し=null / 交差するまでの距離=交差有り
        /// BoundingBoxを前もって設定する。
        /// </summary>
        /// <param name="graphics"></param>
        /// <param name="view"></param>
        /// <param name="proj"></param>
        /// <param name="mouseX"></param>
        /// <param name="mouseY"></param>
        /// <returns></returns>
        virtual public float? ピック判定(   GraphicsDeviceManager graphics,
                                            Matrix view, Matrix proj,
                                            int mouseX, int mouseY )
        {
            // マウスカーソル位置からのレイを作成
            Vector3 nearsource = new Vector3( (float)mouseX, (float)mouseY, 0f );
            Vector3 farsource  = new Vector3( (float)mouseX, (float)mouseY, 1f );

            Matrix world = m_単位行列 * Matrix.CreateTranslation(m_pos);

            Vector3 nearPoint = graphics.GraphicsDevice.Viewport.Unproject( nearsource, proj, view, world );
            Vector3 farPoint  = graphics.GraphicsDevice.Viewport.Unproject( farsource,  proj, view, world );

            // Create a ray from the near clip plane to the far clip plane.
            Vector3 direction = farPoint - nearPoint;

            Ray pickRay = new Ray(nearPoint, direction);

            Nullable<float> result = pickRay.Intersects(m_BoundingBoxA);

            return result;  // 交差有りの場合:交差するまでの距離
        }
        // --------------------------------------




        // --------------------------------------
        /// <summary>
        /// レイとバウンディングボックスの交差判定
        /// 戻り値 : 交差無し=null / 交差するまでの距離=交差有り
        /// BoundingBoxを前もって設定する。
        /// </summary>
        /// <param name="ray"></param>
        /// <returns></returns>
        virtual public float? レイ判定(Ray ray)
        {
            /*
            Matrix M11 - M44
            11 12 13 14 -X
            21 22 23 24 -Y
            31 32 33 34 -Z
            41 42 43 44
            Matrix m = Matrix.CreateTranslation( new Vector3(777,888,999) ); // M31,M32,M33に入る
             */

            Matrix pos = Matrix.CreateTranslation(ray.Position);
            Matrix dir = Matrix.CreateTranslation(ray.Direction);

            Matrix inv = Matrix.Invert(m_単位行列);

            pos *= Matrix.CreateTranslation(-m_pos) * inv;
            dir *= inv;

            Ray wkRay = new Ray(pos.Translation, dir.Translation);// pos,方向

            Nullable<float> result = wkRay.Intersects(this.m_BoundingBoxA);

            return result;  // 交差有りの場合:交差するまでの距離
        }
        // --------------------------------------


        /// <summary>
        /// BoundingFrustumとBoundingBoxの交差判定
        /// 戻り値:交差有り:true
        /// </summary>
        /// <param name="m"></param>
        /// <returns></returns>
        virtual public bool すい台判定(Matrix ViewProj)
        {
            // Matrix wkM = Matrix.Multiply( Matrix.Invert(m_単位行列), Matrix.CreateTranslation(m_pos) ); // 誤り
            Matrix wkM = Matrix.Multiply( m_単位行列, Matrix.CreateTranslation(m_pos) );

            BoundingFrustum bf = new BoundingFrustum( Matrix.Multiply( wkM, ViewProj ) );

            return bf.Intersects( this.m_BoundingBoxA );
        }













    }
}
